from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindSourcesOutput(TcBaseObj):
    """
    Contains multiple source BOMLine and integer based index to point output to corresponding input.
    
    :var inputIndex: Index to input list. Useful to map output list to input list.
    :var sourceLines: The source BOMLines objects for input target BOMLine and source BOMWindow. Source BOMLine and
    Target BOMLine are associated via PublishLink.
    """
    inputIndex: int = 0
    sourceLines: List[BOMLine] = ()


@dataclass
class FindSourcesResponse(TcBaseObj):
    """
    Contains 'FindSourcesOutput' containing multiple source BOMLine and index to map to source BOMLine to corresponding
    input target BOMLine.
    
    :var output: 'FindSourcesOutput' containing multiple source BOMLine and integer index to map source BOMLine to
    input target BOMLine.
    :var serviceData: 'ServiceData' with plain objects containing multiple source BOMLine and partial errors.
    """
    output: List[FindSourcesOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2008_03.Session import FavoritesList
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetShortcutsResponse(TcBaseObj):
    """
    A structure contains Favorites hierarchy and map of LHNSectionComponent given section names.
    
    :var favorites: A hierarchical Favorites tree structure list that contains all the favorites containers and
    favorites objects for the current session user. The Favorites section can be populated only if FavoritesSection is
    passed as a key in the input map to this API.
    :var shortcuts: A map structure that includes the given section name and corresponding content of the section. The
    key is the name of the section and the value is the LHNSectionComponent structure, which has a Teamcenter object
    and a nonTeamcenter object . The preferred keys allowed in this map are MyFavorites, HistoryList,
    QuickLinksSection,  MyQuicklinkSection etc.
    :var serviceData: ServiceData contains any failures and Teamcenter objects  wrapped in LHNSecitonComponents and
    default properties. The following errors may be encountered:
    1700 (error code):  The preference cannot be found.
    515024 ( error code): The given tag does not exist in the database or is not a persistent object tag.
    """
    favorites: FavoritesList = None
    shortcuts: LHNSectionComponentsMap = None
    serviceData: ServiceData = None


@dataclass
class LHNNonTcObjectSectionComponent(TcBaseObj):
    """
    The sections in the left hand navigation can contain either Teamcenter objects or non-Teamcenter 
    objects.
    Sections which contain Teamcenter objects are 'Open Items', 'History', etc and the sections which 
    contain non-Teamcenter bjects are 'I Want To' section. This structure in turn contains the
    LHNSectionComponentDetails which maintains additional information pertaining to the object.
    
    Example:
    Consider that the 'I Want To' section in the left hand navigation contains the command 'Create an 
    Item'.
    This structure would contain the below details in the LHNSectionComponentDetails.
    Key = ActionName, Value = newItemAction
    Key = DisplayName, Value = Create an Item
    Key = CommandID, Value = com.teamcenter.rac.newItem
    
    Consider that the 'QuickLinkSection' section in the left hand navigation contains the command 'like 
    a 'home folder'  in quick links.
    key = QuickLinkId
    
    
    :var nonTcObjects: Non Tc Objects like Home, folder etc
    """
    nonTcObjects: LHNSectionComponentDetails = None


@dataclass
class LHNSectionComponents(TcBaseObj):
    """
    The purpose of this structure is to give a single name to 'LHNNonTcObjectSectionComponentMap' and 
    'LHNTcObjectSectionComponentMap' which is section component. Each section component is puted into 
     LHNSectionComponentsMap.
    
    LHNNonTcObjectSectionComponentMap: It is a map for nontcobject , whose key is 'int' using for 
    placeHolder and value are NonTcObjectSection.
    
    LHNTcObjectSectionComponentMap key is for placeholder and value as tccomponent object.
    
    :var nonTcObjects: nonTcObjects
    :var tcObjects: tcObjects
    """
    nonTcObjects: LHNNonTcObjectSectionComponentMap = None
    tcObjects: LHNTcObjectSectionComponentMap = None


@dataclass
class LHNTcObjectSectionComponent(TcBaseObj):
    """
    The purpose of this structure is to represents the Teamcenter object details, Which is get use in 
    the Left Hand Navigation pane of the clients. Which is either a tcobject or non tcobject.
    
    Teamcenter::BusinessObject  This is a TcComponent which have all the bussiness property of  
    teamcenter object.
    e.g Items in History section, Items in OpenItemSection etc.
                 
    LHNSectionComponentDetails It must contain the detail information about the nonTcObject at which 
    place we have to insert and what are the action name and display name  associated with it.
    e.g LHNSectionComponentDetails('Action Name',  newItemAction );
        LHNSectionComponentDetails('Display Name',  Create an Item );
    
    
    :var tcObject: Specifies the Teamcenter object
    :var details: Additional details of the section component.
    """
    tcObject: BusinessObject = None
    details: LHNSectionComponentDetails = None


@dataclass
class MultiPreferenceResponse2(TcBaseObj):
    """
    The structure used to get the returned preference
    
    :var data: The successful Object ids, partial errors and failures
    :var preferences: List of ReturnedPreferences2 Object
    """
    data: ServiceData = None
    preferences: List[ReturnedPreferences2] = ()


@dataclass
class ReturnedPreferences2(TcBaseObj):
    """
    This is the structure which is used to define the information for one preference.
    
    :var scope: The scope of the preference, "all", "site", "user", "group", or "role".
    :var category: The variable to hold the category name
    :var description: The variable to hold the description
    :var prefType: Preference Type
    :var isArray: Preference Array
    :var isDisabled: Is the preference disabled
    :var values: The value of the preference
    :var name: The name of the preference
    """
    scope: str = ''
    category: str = ''
    description: str = ''
    prefType: int = 0
    isArray: bool = False
    isDisabled: bool = False
    values: List[str] = ()
    name: str = ''


"""
This map is used to maintain the place holder for nonTcObject.

LHNNonTcObjectSectionComponent is a structure in which there is a map element haveing information 
about the non TcObject. Such as for 'IWanToSection' it is 'Action Name' 'Display Name' and 
commandID.
 
Example:                                  
LHNNonTcObjectSectionComponentMap( 1, 'create an item')
LHNNonTcObjectSectionComponentMap( 2, 'create a Dataset')
"""
LHNNonTcObjectSectionComponentMap = Dict[int, LHNNonTcObjectSectionComponent]


"""
This is a map containing the details of a object in the left hand navigation.
The keys vary based on the type of the section name.
Example
For IWantToSection keys are ActionName, DisplayName,  commandID.

Action Name = newItemAction
Display Name = Create an Item
LHNSectionComponentDetails('Action Name',  newItemAction );
LHNSectionComponentDetails('Display Name',  Create an Item );
LHNSectionComponentDetails('Display Name',  com.teamcenter.rac.newItem )
"""
LHNSectionComponentDetails = Dict[str, str]


"""
In this map key is the SectionName and value is the LHNSectionComponent structure, which have 
TcObject and nonTcObject as it's element.
The prefered Keys allowed in this map are MyFavorites, HistoryList, QuickLinksSection, 
MyQuicklinkSection etc.
  
Example   LHNSectionComponentsMap ('HistorySection' , SectionComponent structure for History 
Section)

"""
LHNSectionComponentsMap = Dict[str, LHNSectionComponents]


"""
This is a input parameter to the getShortcuts operation. This is a map where the key represents the 
section name in the left hand navigation 
and the value represents the preference name that needs to be looked up.
Valid key value pairs are:
Key = QuickLinksSection, Value = MyTeamcenterQuicklinksection
Key = FavoritesSection, value = My Favorites

Valid keys in the map are
QuickLinksSection, HistorySection, FavoritesSection, IWantToSection, OpenItemsSection.
Valid values against these keys in the map are
MyFavorites, HistoryList, QuickLinksSection, MyQuicklinkSection, OpenItemSection etc.

"""
LHNShortcutInputs = Dict[str, str]


"""
This map is used to maintain the place holder for the sectioncomponent.Which is either a tcObject or 
nontcobject.


LHNTcObjectSectionComponent It is simply a collection of TcObject or nonTcObject.
             
Example:
LHNTcObjectSectionComponentMap(0, any tcobject or  nontcobject);
LHNTcObjectSectionComponentMap(1, any tcobject or  nontcobject);
"""
LHNTcObjectSectionComponentMap = Dict[int, LHNTcObjectSectionComponent]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision, RevisionRule, TransferMode
from tcsoa.gen.GlobalMultiSite._2010_04.ImportExport import ExportObjectsToPLMXMLResponse, ImportObjectsFromPLMXMLResponse
from typing import List
from tcsoa.gen.GlobalMultiSite._2007_12.ImportExport import NamesAndValues
from tcsoa.utils import TcService


class ImportExportService(TcService):

    @classmethod
    def importObjectsFromPLMXML(cls, xmlFileTicket: str, namedRefFileTickets: List[str], transfermode: TransferMode, icRev: ItemRevision, sessionOptions: List[NamesAndValues]) -> ImportObjectsFromPLMXMLResponse:
        """
        The importObjectsFromPLMXML operation will import the objects from a PLMXML file. The XML file and the named
        reference files for datasets should be uploaded to transient volume before the calling of this operation. User
        can use getTransientFileTicketsForUpload operation from core.FileManagementService to generate the ticket and
        then call putFileViaTicket operation from soa.client.FileManagementUtility to perform the file upload to
        transient volume.
        
        Use cases:
        Use Case 1: Import PLMXML file to database
        You can import PLMXML file by specify:
        1)    The xml file that you want to import.
        2)    The related dataset file you want to import.
        3)    Transfer mode that you want to traverse to the xml.
        4)    The incremental change context.
        5)    Session options.
        
        
        Exceptions:
        >203474    will throw while the input file name is too long.
        Other PIE failure in the whole import process.
        """
        return cls.execute_soa_method(
            method_name='importObjectsFromPLMXML',
            library='GlobalMultiSite',
            service_date='2010_04',
            service_name='ImportExport',
            params={'xmlFileTicket': xmlFileTicket, 'namedRefFileTickets': namedRefFileTickets, 'transfermode': transfermode, 'icRev': icRev, 'sessionOptions': sessionOptions},
            response_cls=ImportObjectsFromPLMXMLResponse,
        )

    @classmethod
    def exportObjectsToPLMXML(cls, exportObjects: List[BusinessObject], transfermode: TransferMode, revRule: RevisionRule, languages: List[str], xmlFileName: str, sessionOptions: List[NamesAndValues]) -> ExportObjectsToPLMXMLResponse:
        """
        The 'exportObjectsToPLMXML' operation will generate a PLMXML file and a export log file for the input object
        list, transfer mode, revision rule and language set.
        
        Use cases:
        Use Case 1: Export object to PLMXML file
        You can export any business object by specify:
        1)    The objects that you want to exported.
        2)    Transfer mode and revision rule.
        3)    Languages that for localization value.
        4)    Xml file name.
        5)    Session options.
        
        
        Exceptions:
        >203331  will throw while all input objects is invalid objects.
        203447  will throw while fail to get download ticket from FMS.
        203486  will throw while the input XML file name contains full file path.
        Other PIE failure in the whole export process.
        """
        return cls.execute_soa_method(
            method_name='exportObjectsToPLMXML',
            library='GlobalMultiSite',
            service_date='2010_04',
            service_name='ImportExport',
            params={'exportObjects': exportObjects, 'transfermode': transfermode, 'revRule': revRule, 'languages': languages, 'xmlFileName': xmlFileName, 'sessionOptions': sessionOptions},
            response_cls=ExportObjectsToPLMXMLResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetTransientTicketsDownloadInput(TcBaseObj):
    """
    GetTransientTicketsDownloadInput structure contains a transient file write ticket for which a read ticket is to be
    obtained. If deleteFlag is true then the file is deleted from temporary storage after it is read.
    
    :var transientFileWriteTicket: Transient file write ticket.
    :var deleteFlag: If true, the file is deleted from temporary storage after it is read.
    """
    transientFileWriteTicket: str = ''
    deleteFlag: bool = False


@dataclass
class GetTransientTicketsDownloadResponse(TcBaseObj):
    """
    GetTransientTicketsDownloadResponse structure contains transient file read tickets and corresponding failure codes
    and error strings.
    
    :var transientFileReadTickets: A list of transient file read tickets.
    :var serviceData: Service Data in which the partial errors are communicated to the client.
    """
    transientFileReadTickets: List[str] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.GlobalMultiSite._2007_12.ImportExport import NamesAndValues, CallToRemoteSiteResponse, OwningSiteAndObjs
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ImportExportService(TcService):

    @classmethod
    def getRemoteSites(cls, siteType: str) -> ServiceData:
        """
        This operation returns a list of sites registered for the requested type of transfer.
        """
        return cls.execute_soa_method(
            method_name='getRemoteSites',
            library='GlobalMultiSite',
            service_date='2007_12',
            service_name='ImportExport',
            params={'siteType': siteType},
            response_cls=ServiceData,
        )

    @classmethod
    def requestExportToRemoteSites(cls, reason: str, sites: List[BusinessObject], objects: List[BusinessObject], optionSet: BusinessObject, optionNameAndValues: List[NamesAndValues], sessionOptionNamesAndValues: List[NamesAndValues]) -> CallToRemoteSiteResponse:
        """
        This operation exports objects to specified sites using Global Multisite.
        
        Use cases:
        This operation gets invoked from RAC when user does following actions from Navigator or structure manager
        1>    Tools->Export->To Remote Site Via Global Services.
        2>    Tools->Export->To PDX
        3>    Tools->Export->To Briefcase
        """
        return cls.execute_soa_method(
            method_name='requestExportToRemoteSites',
            library='GlobalMultiSite',
            service_date='2007_12',
            service_name='ImportExport',
            params={'reason': reason, 'sites': sites, 'objects': objects, 'optionSet': optionSet, 'optionNameAndValues': optionNameAndValues, 'sessionOptionNamesAndValues': sessionOptionNamesAndValues},
            response_cls=CallToRemoteSiteResponse,
        )

    @classmethod
    def requestImportFromRemoteSites(cls, reason: str, owningSitesAndObjs: List[OwningSiteAndObjs], optionSet: BusinessObject, optionNamesAndValues: List[NamesAndValues], sessionOptionNamesAndValues: List[NamesAndValues]) -> CallToRemoteSiteResponse:
        """
        This operation imports objects to specified sites using Global Multisite.
        
        Use cases:
        This operation is not in use.
        """
        return cls.execute_soa_method(
            method_name='requestImportFromRemoteSites',
            library='GlobalMultiSite',
            service_date='2007_12',
            service_name='ImportExport',
            params={'reason': reason, 'owningSitesAndObjs': owningSitesAndObjs, 'optionSet': optionSet, 'optionNamesAndValues': optionNamesAndValues, 'sessionOptionNamesAndValues': sessionOptionNamesAndValues},
            response_cls=CallToRemoteSiteResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Translation._2007_06.TranslationManagement import UpdateTranslationRequestResponse, QueryTranslationRequestsResponse, InsertDatasetVersionArgs, QueryTranslationRequestsArgs, CreateDatasetOfVersionArgs, CreateDatasetOfVersionResponse, InsertDatasetVersionResponse, UpdateTranslationRequestArgs
from typing import List
from tcsoa.utils import TcService


class TranslationManagementService(TcService):

    @classmethod
    def insertDatasetVersion(cls, inputs: List[InsertDatasetVersionArgs]) -> InsertDatasetVersionResponse:
        """
        Insert a Dataset at a specific version.
        
        Use cases:
        When creating Datasets in the system, sometimes the versions are to be specified.  This is an important detail
        to maintain, especially during migration of external data into Teamcenter from another system.  This operation
        provides the ability to insert a Dataset at a particular version within the system which is different than the
        standard behavior provided by the Data Management Services.
        """
        return cls.execute_soa_method(
            method_name='insertDatasetVersion',
            library='Internal-Translation',
            service_date='2007_06',
            service_name='TranslationManagement',
            params={'inputs': inputs},
            response_cls=InsertDatasetVersionResponse,
        )

    @classmethod
    def queryTranslationRequests(cls, inputs: List[QueryTranslationRequestsArgs]) -> QueryTranslationRequestsResponse:
        """
        Query for ETSTranslationRequest  within the Teamcenter database
        
        Use cases:
        Instead of using saved queries to find ETSTranslationRequest objects in the system, you would use this method
        which is specific to finding ETSTranslationRequest objects.
        """
        return cls.execute_soa_method(
            method_name='queryTranslationRequests',
            library='Internal-Translation',
            service_date='2007_06',
            service_name='TranslationManagement',
            params={'inputs': inputs},
            response_cls=QueryTranslationRequestsResponse,
        )

    @classmethod
    def updateTranslationRequest(cls, inputs: List[UpdateTranslationRequestArgs]) -> UpdateTranslationRequestResponse:
        """
        Update an ETSTranslationRequest object within the Teamcenter database.  Only a few properties of the
        ETSTranslationRequest can be modified using this service.
        
        Use cases:
        When changes need to be made to an ETSTranslationRequest, this method will allow you to make changes to some
        fields of the ETSTranslationRequest object.
        """
        return cls.execute_soa_method(
            method_name='updateTranslationRequest',
            library='Internal-Translation',
            service_date='2007_06',
            service_name='TranslationManagement',
            params={'inputs': inputs},
            response_cls=UpdateTranslationRequestResponse,
        )

    @classmethod
    def createDatasetOfVersion(cls, inputs: List[CreateDatasetOfVersionArgs]) -> CreateDatasetOfVersionResponse:
        """
        Creates a new Dataset with a particular version.
        
        Use cases:
        If a Dataset does not already exist within Teamcenter and the application needs to create one at a particular
        version, this operation provides that ability.  This is a key factor when migrating data from other systems
        where the versions need to stay in sync for validation purposes.
        """
        return cls.execute_soa_method(
            method_name='createDatasetOfVersion',
            library='Internal-Translation',
            service_date='2007_06',
            service_name='TranslationManagement',
            params={'inputs': inputs},
            response_cls=CreateDatasetOfVersionResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceVis._2015_03.MassiveModelVisualization import GroupOccsByPropertyInput, GetStructureFilesResponse, OccsGroupedByPropertyResponse, ProductStructureIdInput, UpdateCollectionInput, GetProductStructureIdResponse, GetIndexedProductsResponse, ProductAndConfigInfoInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class MassiveModelVisualizationService(TcService):

    @classmethod
    def getIndexedProducts(cls) -> GetIndexedProductsResponse:
        """
        This operation retrieves the information about the product and configurations whose product structure
        information is indexed. The product and configuration information are returned only for products which have
        been enabled for massive model visualization(MMV).
        
        Use cases:
        The Quicksilver Data server(QDS) is deployed on a LAN closer to the visualization client. The QDS serves the
        product structure information to visualization clients connecting to it. The QDS is configured to periodically
        communicate to the Tcserver and retrieve the indexed product structure information.
        1.    QDS boots and invokes getIndexedProducts and gets the indexed products.
        2.     Invokes getStructureFiles operation on each of the indexed product to get the full product structure
        information represented in a binary file(*.mmp).
        3.    Periodically invokes getStructureFiles operation with previous delta identifier token to retrieve the
        changed product structure information.
        """
        return cls.execute_soa_method(
            method_name='getIndexedProducts',
            library='Internal-ActiveWorkspaceVis',
            service_date='2015_03',
            service_name='MassiveModelVisualization',
            params={},
            response_cls=GetIndexedProductsResponse,
        )

    @classmethod
    def getStructureFiles(cls, productAndConfigInfoInput: List[ProductAndConfigInfoInput]) -> GetStructureFilesResponse:
        """
        This operation retrieves the full or delta product structure information for the given product and
        configuration. The ticket to the file(s) that contain the  product structure information is returned via this
        operation. 
        When this operation is invoked by the client to get the full product structure file, the response may contain a
        single complete product structure file or a single complete product structure file and a set of delta product
        structure files. In case a complete and a set of files containing delta changes are sent then the receiving
        client has to merge the delta files into the complete structure file.
        When this operation is invoked by the client to get the delta product structure, the response may contain a set
        of delta product structure files or a single complete product structure file and a set of delta product
        structure files.  In case where only a set of delta files are returned, the client need to merge those file
        with its available complete structure file. When a single complete product structure file and a set of delta
        product structure files are returned, then this means that the delta token identifier could not be identified
        on the server and hence a complete product structure information has been returned.
        The response also contains a delta identifier token. This delta identifier token has to be passed during the
        next call to  getStructureFiles operation in case the client needs the delta files since the previous call to 
        getStructureFiles operation.
        
        
        Use cases:
        The Quicksilver Data server(QDS) is deployed on a LAN closer to the visualization client. The QDS acts as a
        product structure server for the visualization clients. The QDS is configured to periodically communicate to
        the Tcserver and retrieve the indexed product structure information.
        1.    QDS boots and invokes getIndexedProducts  and gets the products that are indexed.
        2.     Invokes getStructureFiles operation on each of the indexed product to get the full product structure 
        information represented in a binary file(*.mmp).
        3.    Since the product structure changes are continuously tracked and product structure file produced, the
        response would contain a complete product structure file along with a set of delta product structure files that
        were generated since the complete product structure file was produced.
        4.    If the product structure information is returned as a complete file and a set of delta files then the
        client merges the delta file into the complete structure to get the current complete product structure.
        5.    The QDS invokes getStructureFiles periodically to get the next delta that represents the product
        structure changes.
        6.    If only delta product structure file is returned then those files are merged with the complete product
        structure file that the client already has.
        """
        return cls.execute_soa_method(
            method_name='getStructureFiles',
            library='Internal-ActiveWorkspaceVis',
            service_date='2015_03',
            service_name='MassiveModelVisualization',
            params={'productAndConfigInfoInput': productAndConfigInfoInput},
            response_cls=GetStructureFilesResponse,
        )

    @classmethod
    def getStructureIdFromRecipe(cls, productStructureIdInput: List[ProductStructureIdInput]) -> GetProductStructureIdResponse:
        """
        This operation retrieves the id that is used to identify a product structure information. The id is a unique
        identifier which gets written into the MMP file as the product Id. The Quicksilver Data server(QDS) maintains
        the product structure information for different product and configurations. Now when the Visualization client
        sends a request to QDS asking information about a product and configuration, the QDS invokes this operation to
        determine the id of the structure which contain information about the requested product and configuration.
        
        Say for example the QDS may have a product structure information with id "UidOfBomIndexAdminData1" that
        contains product structure information of product1 for RevisionRule as "Latest Working" and VariantRule
        "vrule1", "vrule2", "vrule3" and a product structure information with id "UidOfBomIndexAdminData2" that
        contains product structure information of product2 for RevisionRule as "Latest Released" and VariantRule
        "vrule1", "vrule2", "vrule3" and. Now if the Visualization client wants to display the product1 for "Latest
        working" with "vrule2", it contacts the QDS server with this information. The QDS in turn  invokes this
        operation with the given information to determine the id of the mmp file, in this case it would return
        "UidOfBomIndexAdminData1" as id in its response.
        
        
        Use cases:
        The Quicksilver Data server(QDS) is deployed on a LAN closer to the visualization client. The QDS acts as a
        product structure server for the visualization clients. The QDS is configured to periodically communicate to
        the Tcserver and retrieve the indexed product structure information.
        1.    QDS boots and invokes getIndexedProducts  and gets the products that are indexed.
        2.     Invokes getStructureFiles operation on each of the indexed product to get the product structure
        information represented in a binary file(*.mmp).
        3.    If the product structure information is returned as a complete file and a set of delta files then the
        client merges the delta file into the complete structure to get the current complete product structure .
        4.    If only delta product structure file is returned then those files are merged with the complete file that
        the client already has.
        5.    A visualization client requests product structure information for a given product and configuration using
        a recipe object. The recipe object could be Awb0ProductContextInfo or 
        VisStructureContext etc. Now QDS invokes getStructureIdFromRecipe to determine which file contains the
        requested product structure information.
        6.    The spatial structure corresponding to the product structure file is then served to the visualization
        client for it to apply  massive model visualization algorithm.
        """
        return cls.execute_soa_method(
            method_name='getStructureIdFromRecipe',
            library='Internal-ActiveWorkspaceVis',
            service_date='2015_03',
            service_name='MassiveModelVisualization',
            params={'productStructureIdInput': productStructureIdInput},
            response_cls=GetProductStructureIdResponse,
        )

    @classmethod
    def groupOccurrencesByProperties(cls, groupOccsByPropertyInputList: List[GroupOccsByPropertyInput]) -> OccsGroupedByPropertyResponse:
        """
        This operation classifies Awb0Element objects which are identified by their PFUID (An identifier used to
        identify a product structure line in ACE index BOM) into groups. In the list of 'GroupOccsByPropertyInput',
        each input contains an internal property name, list of property values in 'PropertyGroupingValue' identifying
        the grouping criteria and a list of PFUIDs that need to be organized into groups. Each 'PropertyGroupingValue'
        input contains a start and an end value. The end value is to be used for range values if populated. 
        
        The response contains the PFUIDs that were grouped based on the property name and values. PFUIDs that could not
        be grouped as per input property names and values are retuned back in a separate list.
        
        
        Use cases:
        Use case1: 
        Coloring the part assembly displayed in Active Workspace by visualization server when rendering in
        non-MMV(Massive Model Visualization) mode .
        1.    In the Active Workspace client, a user wants to visualize an assembly.
        2.    The color of the parts displayed in the visualization view has to match the colors on the bar chart for
        the objects in the search results.
        3.    The AW visualization client invokes the visualization server with the criteria that contain the color
        code to group the rendered objects based on the property name and property values.
        4.    The AW visualization server initially renders all the current objects it has in its view in a see through
        mode
        5.    The AW visualization server now invokes 'Teamcenter::Soa::Query::_2014_11::Finder::
        groupObjectsByProperties' and passes the property name, property values and the objects that it is currently
        displaying.
        6.    AW visualization server on receiving the objects grouped based on property names and values, renders the
        parts based on the colors associated to the group.
        
        Use case2: 
        Coloring the part assembly displayed in Active Workspace by visualization server when rendering in MMV(Massive
        Model Visualization) mode .
        1.    In the Active Workspace client, a user wants to visualize an assembly.
        2.    The color of the parts displayed in the visualization view has to match the colors on the bar chart for
        the objects in the search results.
        3.    The AW visualization client invokes the visualization server with the criteria that contain the color
        code to group the rendered objects based on the property name and property values.
        4.    The AW visualization server initially renders all the current objects it has in its view in a see through
        mode
        5.    The AW visualization server since it is rendering in MMV mode does not have Business Objects instead has
        the PFUID of the parts. The AW visualization server now invokes 'groupOccurrencesByProperties' and passes the
        property name, property values, and the PFUID of the objects that it is currently displaying.
        6.    AW visualization server on receiving the PFUID grouped based on property names and values, renders the
        parts based on the colors associated to the group.
        """
        return cls.execute_soa_method(
            method_name='groupOccurrencesByProperties',
            library='Internal-ActiveWorkspaceVis',
            service_date='2015_03',
            service_name='MassiveModelVisualization',
            params={'groupOccsByPropertyInputList': groupOccsByPropertyInputList},
            response_cls=OccsGroupedByPropertyResponse,
        )

    @classmethod
    def updateDeltaCollection(cls, updateCollectionInput: List[UpdateCollectionInput]) -> ServiceData:
        """
        This operation updates the Awv0MMPDeltaCollection dataset with the file that contains product structure changes.
        
        Use cases:
        The system administrator sets up the runTcFTSIndexer to run periodically.
        1.    The TcFtsIndexer finds all the Awb0BOMIndexAdminData that contains product and configuration whose
        product structure  need to be indexed.
        2.     Invokes processBomIndex operation which returns the product structure information in TcXML format.
        3.    TcXML file is transformed into SOLR schema and uploaded into SOLR.
        4.    TcXML file is also transformed into MMP format using the tcxml2mmp converter utility.
        5.    An upload ticket is obtained for the MMP file using the getRegularFileTicketsForUpload operation.
        6.    The file is uploaded to Teamcenter volume using FCC.
        7.    The uploaded file is committed to Teamcenter as ImanFile object using commitRegularFiles operation.
        8.    The ImanFile object is now associated to Awv0MMPDeltaCollection using updateDeltaCollection.
        """
        return cls.execute_soa_method(
            method_name='updateDeltaCollection',
            library='Internal-ActiveWorkspaceVis',
            service_date='2015_03',
            service_name='MassiveModelVisualization',
            params={'updateCollectionInput': updateCollectionInput},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.utils import TcService
from tcsoa.gen.Internal.ProjectManagement._2010_04.ScheduleManagement import TranslationDataContainer


class ScheduleManagementService(TcService):

    @classmethod
    def translateFour(cls, scheduleUid: str) -> TranslationDataContainer:
        """
        Translation Data Container
        """
        return cls.execute_soa_method(
            method_name='translateFour',
            library='Internal-ProjectManagement',
            service_date='2010_04',
            service_name='ScheduleManagement',
            params={'scheduleUid': scheduleUid},
            response_cls=TranslationDataContainer,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Cad._2010_04.DataManagement import ReviseResponse, SaveAsNewItemInfo, ReviseInfo, SaveAsNewItemResponse
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def revise(cls, info: List[ReviseInfo]) -> ReviseResponse:
        """
        Revises all the ItemRevision  objects given in the input vector and propagation of their relations via deep
        copy input. The client has the option to supply a set of deep copy rules. Default deep copy rules reside in the
        system. The client may retrieve them using the 'getDeepCopyInfo'  operation and change or add to the list for
        input to this operation as desired. If the client does not supply any deep copy rules then the default ones
        will be used. If the client supplies deep copy rules, then none of the default rules will be used. If the
        client provides new property values for the master form in the input then these will be applied to the newly
        created ItemRevision objects master form.
        
        Use cases:
        User has an iitem revision that is moving from the design phase to the production phase. The final design needs
        to be maintained unaltered so a new revision of the item revision is made that can be updated per production.
        When revising the item revision, the client can also specify deep copy behavior to determine what is done with
        attached objects (such as datasets and forms  ). The client can reference the original objects, make new copies
        of the objects, or not have a reference at all.  The client can also specify new attributes for the attached
        object.
        """
        return cls.execute_soa_method(
            method_name='revise',
            library='Internal-Cad',
            service_date='2010_04',
            service_name='DataManagement',
            params={'info': info},
            response_cls=ReviseResponse,
        )

    @classmethod
    def saveAsNewItem(cls, info: List[SaveAsNewItemInfo]) -> SaveAsNewItemResponse:
        """
        This service will create a new item that is based on the original input item.  The client has the option to
        supply a set of deep copy rules.  Default deep copy rules reside in the system, the client may retrieve them
        using 'getDeepCopyInfo' operation and change or add to the list for input to this operation as desired. If the
        client does not supply any deep copy rules then the default ones will be used. If the client supplies deep copy
        rules, then none of the default rules will be used. If the client provides new property values for the master
        form in the input then these will be applied to the newly created item and  item revision master forms.
        
        Use cases:
        The client has an existing item that they want to preserve yet continue making improvements. The client can
        specify a new item type when doing the save and save the new item as a different item type than the original.
        The client can also specify deep copy behavior to determine what is done with attached objects (such as
        datasets and forms ). The client can reference the original objects, make new copies of the objects or not have
        a reference at all.  The client can also specify new attributes for the attached forms.
        """
        return cls.execute_soa_method(
            method_name='saveAsNewItem',
            library='Internal-Cad',
            service_date='2010_04',
            service_name='DataManagement',
            params={'info': info},
            response_cls=SaveAsNewItemResponse,
        )



from __future__ import annotations

from tcsoa.gen.Classification._2007_01.Classification import ClassificationObject
from typing import List
from tcsoa.gen.Internal.Classification._2018_11.Classification import SaveClassificationObjectsResponse
from tcsoa.utils import TcService


class ClassificationService(TcService):

    @classmethod
    def saveClassificationObjects(cls, classificationObjects: List[ClassificationObject]) -> SaveClassificationObjectsResponse:
        """
        Creates or updates one or more classification objects and (optionally) attach them to a WorkspaceObject object,
        thus classifying it. When the Classification objects are not associated with any WorkspaceObject they act as
        standalone classification objects. A classification object is also called an ICO object. This supports
        traditional ICO.
        
        Use cases:
        User wants to classify a Workspace Object or create a standalone classification object (ICO) or update an
        existing classification object, in a traditional class. This operation can be combined with other operations
        like createItems() to create workspace object and then associate the workspace object to the classification
        object. Before creating a classification object, a classification class hierarchy should already be created by
        the classification admin user in Teamcenter. This hierarchy should include a storage class (a class that allows
        instances to be created and associated to it) for which the classification objects need to be created. Values
        of any attributes associated with classification objects can also be populated.
        """
        return cls.execute_soa_method(
            method_name='saveClassificationObjects',
            library='Internal-Classification',
            service_date='2018_11',
            service_name='Classification',
            params={'classificationObjects': classificationObjects},
            response_cls=SaveClassificationObjectsResponse,
        )



from __future__ import annotations

from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CheckinInput(TcBaseObj):
    """
    Input for the checkin service.
    
    :var ciProcess: true if "process" opition is selected.
    :var isRecursive: true if "isRecursive" opition is selected.
    """
    ciProcess: bool = False
    isRecursive: bool = False


@dataclass
class CheckoutInput(TcBaseObj):
    """
    Input for the checkout service.
    
    :var coProcess: true if "process" opition was selected.
    :var isRecursive: true if "isRecursive" opition was selected.
    :var coTools: true if "coTools" opition was selected.
    :var coPlant: true if "coPlant" opition was selected.
    :var coAssembly: true if "coAssembly" opition was selected.
    :var changeId: The user Id.
    :var reason: The reason for the checkout.
    """
    coProcess: bool = False
    isRecursive: bool = False
    coTools: bool = False
    coPlant: bool = False
    coAssembly: bool = False
    changeId: str = ''
    reason: str = ''



from tcsoa.gen.ActiveWorkspaceBom._2015_10.services import OccurrenceManagementCadService as imp0
from tcsoa.utils import TcService


class OccurrenceManagementCadService(TcService):
    createBOMWindowsFromBookmarks = imp0.createBOMWindowsFromBookmarks
    createBOMWindowsFromContexts = imp0.createBOMWindowsFromContexts



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Validation._2012_02.Validation import ValidationResultsResponse2, ValidationResultInfo2, ResultOverrideApproval
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ValidationService(TcService):

    @classmethod
    def performActionOnOverrideApproval(cls, inputs: List[ResultOverrideApproval]) -> ServiceData:
        """
        This operation will perform the specified result override approval life cycle actions for given
        ValidationResult objects. This service will allow a user to create, edit, and/or delete validation result
        override requests, and will also allow designated privileged users to approve or reject override requests.
        
        Use cases:
        It is desirable that requests for override approval on failed checkers to be left to the design user
        discretion. The design user may have good justification to argue that a failed checker can be corrected at a
        later design stage. Therefore, the capability to approve a result override request on a failed checker by
        designated approvers in a company will be necessary to allow the part design to progress smoothly without
        passing an overridden failed checker.
        """
        return cls.execute_soa_method(
            method_name='performActionOnOverrideApproval',
            library='Internal-Validation',
            service_date='2012_02',
            service_name='Validation',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def createOrUpdateValidationResults2(cls, inputs: ValidationResultInfo2) -> ValidationResultsResponse2:
        """
        This service will create new ValidationResult objects or update existing ValidationResult objects.
        ValidationResult objects are created or updated after a validation application executes the specified
        validation business logic to validate against the specified validation target objects to ensure a validation
        target meets compliance.
        
        Use cases:
        The processes of a customer site may require that a part will be released to production only after the part
        meets compliance specified by business logic that is embedded with various validation checkers for certain
        validation application utility. After executing validation business logic on the part, the user will use this
        service to create new ValidationResult objects or update existing ValidationResult objects to demonstrate the
        compliance status of the part.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateValidationResults2',
            library='Internal-Validation',
            service_date='2012_02',
            service_name='Validation',
            params={'inputs': inputs},
            response_cls=ValidationResultsResponse2,
        )



from __future__ import annotations

from tcsoa.gen.Core._2014_06.DigitalSignature import VoidSignaturesInputData, GetSignatureMessagesResponse, ApplySignaturesInputData
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from tcsoa.gen.Core._2011_06.DataManagement import TraceabilityInfoInput
from tcsoa.gen.Core._2012_10.DataManagement import TraceabilityInfoInput1
from tcsoa.gen.Core._2014_06.DataManagement import TraceabilityReportOutput2, TraceabilityReportOutputLegacy


class DigitalSignatureService(TcService):

    @classmethod
    def getSignatureMessages(cls, targetObject: List[BusinessObject]) -> GetSignatureMessagesResponse:
        """
        This operation returns signature messages for a list of business objects. These signature messages are used by
        SOA framework method  com.teamcenter.soa.client.PKCS7.sign     API to generate CMS string (Cryptographic
        Message Syntax). The operation response GetSignatureMessagesResponse contains details of signature messages
        computed for each of the input business objects along with the ServiceData. The attributes that are to be used
        for signature message computation are configured using the business object constant
        Fnd0DigitalSignatureAttributes and Fnd0DigitalSignatureChildObjects.
        """
        return cls.execute_soa_method(
            method_name='getSignatureMessages',
            library='Core',
            service_date='2014_06',
            service_name='DigitalSignature',
            params={'targetObject': targetObject},
            response_cls=GetSignatureMessagesResponse,
        )

    @classmethod
    def voidSignatures(cls, input: List[VoidSignaturesInputData], electronicSignature: str) -> ServiceData:
        """
        This operation voids the selected digital signatureson a given target object.. The details of  the electronic
        signature may be obtained by calling the requisite SOA framework method com.teamcenter.soa.client.PKCS7.sign.
        ..This is provided as input to the voidDigitalSignatures opearation along with the other inputs. Successful
        completion of the operation, is an indication that the selected digital signature objects have been voided for
        the input business object.
        
        Use cases:
        After applying a digital signature,the object is locked for modification and users would not be able to modify
        any of the attribute values. In certain conditions, the current values on the object would need to be updated
        and the digital signature would need to be reapplied with the updated set of values. To achieve this, the
        existing digital signatures on the object are voided and  required values are updated . After all the updates
        are complete,  digital signature is reapplied on the object. It is to be noted that if all the Digital
        Signatures on an object are voided, then the object state is equivalent to not having any digital signature
        applied and is open for updates 
        """
        return cls.execute_soa_method(
            method_name='voidSignatures',
            library='Core',
            service_date='2014_06',
            service_name='DigitalSignature',
            params={'input': input, 'electronicSignature': electronicSignature},
            response_cls=ServiceData,
        )

    @classmethod
    def applySignatures(cls, input: List[ApplySignaturesInputData]) -> ServiceData:
        """
        This operation applies digital signature to a list of Business objects provided in the input. The operation
        input is a list of DigitalSignatureInput structures. Each structure in this list consists of details pertaining
        to the Business Object and its corresponding CMS (Cryptographic Message Syntax). Digital Signature is allowed
        to be applied on business objects for which the business object constant Fnd0AllowDigitalSignature is enabled.
        
        Use cases:
        To apply digital signature from RAC, the operation getSignatureMessages should first be called to compute the
        signature messages for the input business objects. This should be followed by a call to the SOA framework
        method com.teamcenter.soa.client.PKCS7.sign , which takes signature message as input and provides the encrypted
        string as output. The encrypted string is passed as input to the operation applyDigitalSignatures. Successful
        completion of the operation, is an indication that the digital signature has been applied to the input business
        object.
        """
        return cls.execute_soa_method(
            method_name='applySignatures',
            library='Core',
            service_date='2014_06',
            service_name='DigitalSignature',
            params={'input': input},
            response_cls=ServiceData,
        )


class DataManagementService(TcService):

    @classmethod
    def getTraceReport2(cls, input: List[TraceabilityInfoInput1]) -> TraceabilityReportOutput2:
        """
        This operation generates a Trace Report for the input objects.  The report will contain information about
        complying as well as defining objects which are connected to input object using FND_TraceLink, or its subtype.
        This operation checks if there is any FND_TraceLink relation starting or ending from input object(s). If
        FND_TraceLink relation exists for input object(s), then it gets the other end of FND_TraceLink relation and
        generates a trace report. 
        
        Trace links can be between following objects:
        1  Between occurrences of an ItemRevision 
        2  Between any two WorkspaceObject.
        
        If scope of search structure is defined for the getting trace report in input of this operation by sending top
        lines of BOMWindow instances, then matching trace link instances within the scope windows will be returned.
        
        If input of this operation is having list of object type names, then object type filter will be applied to
        target objects of trace link.
        
        If input of this operation is having list of trace link type names, then those types of trace link will be
        returned in trace report.
        
        If property filter is given in the input of this operation, then the additional filter of property will be
        applied on the output before sending to client.
        
        Trace report tree will be sorted for given property, sort direction can also be defined, if not defined then it
        will get default sorted in ascending direction.
        
        The output of this operation can be either sent to rich client to build the report or to MSExcel application.
        
        User can export this trace report to MSExcel application by sending appropriate exportTo mode in input. If the
        mode of export is "TraceReportMSExcelExport", then trace report will be exported to .xlsm file and this file
        ticket will be sent to rich client. Then rich client will download the file and open MSExcel application.
        
        
        Use cases:
        Suppose user created trace link between Requirement R1 as start point and R2 as end point and creates trace
        link from Requirement R3 as start and R1 as end point.
        When user runs traceability report on R1 requirement he will get R2 object as complying object and R3 will come
        as defining object.
        
        If filter will be added to show only Paragraph type objects, then nothing will be returned in Trace Report as
        the type is not matching with filter.
        
        If filter will be applied to a subtype of FND_TraceLink and above trace link is of type FND_TraceLink, then
        also empty trace report will be returned, as trace link type does not match with filter trace link type vector
        from operation input.
        
        If user invokes command to export the trace report to Excel, then trace report for Requirement R1 will be
        generated and exported in .xlsm file and opened in MSExcel application.
        
        Trace link on occurences:
        Suppose user created trace link between Requirement R1 as start point and Trace Link on occurrence on part P1
        as end point by setting the P1's parent line as context line then this SOA will also return the
        PSBomViewRevision which was set as context line while creating the Trace Link.
        """
        return cls.execute_soa_method(
            method_name='getTraceReport2',
            library='Core',
            service_date='2014_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=TraceabilityReportOutput2,
        )

    @classmethod
    def getTraceReportLegacy(cls, input: TraceabilityInfoInput) -> TraceabilityReportOutputLegacy:
        """
        This operation generates a Trace Report for the input objects.  This operation returns information about
        complying as well as defining objects which are connected to selected object using FND_TraceLink or its subtype
        of GRM relation.
        
        Trace links can be between following objects:
        1.    Between occurrences of an ItemRevision
        2.    Between any two WorkspaceObject.
        
        If indirect trace report flag is set to true during this operation, then user will get trace report for
        ItemRevision if selected object is occurrence, and trace report for Items if selected objects is ItemRevision
        in addition to direct trace report for the selected object.
        
        If trace link is on occurrence then This SOA version will return PSBOMViewRevision context line information
        also.
        
        
        Use cases:
        Suppose user created trace link between Requirement R1 as start point and R2 as end point and creates trace
        link from Requirement R3 as start and R1 as end point.
        When user runs traceability report on R1 requirement he will get R2 object as complying object and R3 will come
        as defining object.
        
        TraceLink on occurrences:
        Suppose user created trace link between Requirement R1 as start point and trace link on occurrence on part P1
        as end point by setting the P1's parent line as context line then this SOA will also return the
        PSBomViewRevision which was set as context line while creating the trace link.
        
        """
        return cls.execute_soa_method(
            method_name='getTraceReportLegacy',
            library='Core',
            service_date='2014_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=TraceabilityReportOutputLegacy,
        )


class ReservationService(TcService):

    @classmethod
    def bulkCancelCheckout(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation cancels a check-out for a set of previously checked-out business objects in bulk. The objects
        will be restored to the pre-check-out state. Only one user can perform a cancel check-out transaction on the
        object if the user has enough privilege on the object. This action may be applied to remote checked-out
        objects, and will cancel the check-out and records the cancel check-out transaction event. Cancel checkout is
        not supported for some of the business objects for e.g. - Item, BOMView,BOMViewRevision, Schedule.
        """
        return cls.execute_soa_method(
            method_name='bulkCancelCheckout',
            library='Core',
            service_date='2014_06',
            service_name='Reservation',
            params={'objects': objects},
            response_cls=ServiceData,
        )

    @classmethod
    def bulkCheckin(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation checks-in a set of previously checked-out business objects in bulk. This operation takes care of
        all complex business logic involved to check-in passed in business objects. Each input object is verified that
        it is locally owned, site owned, and not transferred to another user after the checkout was performed. This
        operation validates precondition defined per type in COTS object and site customization and performs basic
        check-in. Dataset, ItemRevision and many other business object types have their own business logic for
        check-in. This operation calls underlying checkin method of those individual objects.
        
        Note: If the business object ItemRevision is checked out using reservation type
        "RES_RESERVE_BULK_WITH_DELAY_DEEP_COPY" and if the given object is not modified after it was checked out using
        this reservation type then the checkin opetation performed on this object will not increase the sequence number
        of the ItemRevision business object
        """
        return cls.execute_soa_method(
            method_name='bulkCheckin',
            library='Core',
            service_date='2014_06',
            service_name='Reservation',
            params={'objects': objects},
            response_cls=ServiceData,
        )

    @classmethod
    def bulkCheckout(cls, objects: List[BusinessObject], comment: str, changeId: str, reservationType: int) -> ServiceData:
        """
        This operation checks out a set of business objects with given comment and, change identifier in bulk fashion.
        Only one user can perform a check-out transaction on the object. The user must have sufficient privilege on the
        object or the checkout will fail. This operation allows for remote check-out and records the check-out
        transaction event. In the case where the reservationType is RES_RESERVE_BULK_WITH_DELAY_DEEP_COPY, this
        operation checks out business objects without creating the backup copy of the reserved objects. The backup copy
        will be created on demand when the reserved object is modified. This operation is faster than the checkout
        operation.
        
        Use cases:
        The object can be reserved to gain exclusive rights so that no other user can modify it while the reserver is
        modifying the given object.
        """
        return cls.execute_soa_method(
            method_name='bulkCheckout',
            library='Core',
            service_date='2014_06',
            service_name='Reservation',
            params={'objects': objects, 'comment': comment, 'changeId': changeId, 'reservationType': reservationType},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetFilteredIPATypeResponse(TcBaseObj):
    """
    resopnse for getFilteredIPAType SOA
    
    :var serviceData: serive data
    :var flat: a vector of processes that their process structure already contain a flat FIPA.
    :var nested: a vector of processes that their process structure already contains a nested FIPA
    :var unset: processes that their process structure doesn't contain any FIPA yet.
    """
    serviceData: ServiceData = None
    flat: List[BusinessObject] = ()
    nested: List[BusinessObject] = ()
    unset: List[BusinessObject] = ()


@dataclass
class DeleteFilteredIPAInputInfo(TcBaseObj):
    """
    Contains the information about deleting filtered IPAs.
    
    :var process: The business object of the process from which filtered IPA needs to be deleted.
    :var isRecursive: Indicates whether all the filtered IPAs in the hierachy of the process should be deleted or just
    one filtered IPA directly under the process should be deleted.
    """
    process: BusinessObject = None
    isRecursive: bool = False



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class DatasetAttrInfo(TcBaseObj):
    """
    This structure stores the criteria to query for datasets.
    
    :var attrName: The attribute name on the Dataset.
    :var attrValues: List of the Dataset attributes
    """
    attrName: str = ''
    attrValues: List[str] = ()


@dataclass
class DatasetFileInfo(TcBaseObj):
    """
    This structure contains the dataset instance and reference file ticket.
    
    :var dataset: The Dataset object.
    :var referenceFileTicket: The file ticket to download the reference file.
    """
    dataset: Dataset = None
    referenceFileTicket: str = ''


@dataclass
class DatasetFileQueryInfo(TcBaseObj):
    """
    Stores the information to query for Dataset reference files.
    
    :var clientID: A unique string used to identify return data elements and partial errors associated with this input
    structure.
    :var datasetType: The Dataset type name
    :var referenceTypes: A list of Dataset reference type names
    :var attrInfo: The criteria to query for Dataset objects
    """
    clientID: str = ''
    datasetType: str = ''
    referenceTypes: List[str] = ()
    attrInfo: List[DatasetAttrInfo] = ()


@dataclass
class DatasetFilesOutput(TcBaseObj):
    """
    This structure stores the Dataset files for the given Dataset type and reference type.
    
    :var clientID:  The unmodified value from the input clientId. This can be used by the caller to indentify this data
    structure with the source input data.
    :var datasetType: The Dataset type name for which the file is being returned.
    :var datasetReferenceFileInfo: The list of Dataset files with their reference type information.
    """
    clientID: str = ''
    datasetType: str = ''
    datasetReferenceFileInfo: List[DatasetReferenceFileInfo] = ()


@dataclass
class DatasetFilesResponse(TcBaseObj):
    """
    The Dataset reference files returned from the server.
    
    :var datasetFilesOutput: The list of Dataset reference files.
    :var serviceData: Standard 'ServiceData' member
    """
    datasetFilesOutput: List[DatasetFilesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DatasetReferenceFileInfo(TcBaseObj):
    """
    This structure stores the dataset files for the reference type for 'DatasetFilesOutput' structure.
    
    :var datasetReferenceTypeName: The Dataset reference type name for which the file is being returned.
    :var datasetFiles: The list of Dataset files.
    """
    datasetReferenceTypeName: str = ''
    datasetFiles: List[DatasetFileInfo] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.BusinessObjects import Schedule
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ItemDemand(TcBaseObj):
    """
    This represents the output structure to store the labor demand per resource.
    
    :var resourceId: Represents the resource id of an individual labor or resource.
    :var costs: Represents the per period cost of an individual resource.
    """
    resourceId: str = ''
    costs: List[PeriodCost] = ()


@dataclass
class PeriodCost(TcBaseObj):
    """
    This represents the output structure to store the cost vasule per period.
    
    :var periodNum: Represents the period number.
    :var cost: Represents the cost calculated per each period.
    """
    periodNum: int = 0
    cost: str = ''


@dataclass
class DemandProfileRequest(TcBaseObj):
    """
    This represents the input structure for calculating DemandProfile data for each schedule.
    
    :var schedule: Input schedule of which DemandProfile data will be calculated.
    :var startPeriod: Start of schedule Period to calculate data (MM-YYYY).
    :var endPeriod: End of schedule Period to calculate data (MM-YYYY).
    :var monthsInPeriod: Number of months in the period.
    :var currency: Currency to calculate the cost (ISO 4217). (Not currently used)
    """
    schedule: Schedule = None
    startPeriod: str = ''
    endPeriod: str = ''
    monthsInPeriod: int = 0
    currency: str = ''


@dataclass
class DemandProfileResponse(TcBaseObj):
    """
    This represents the output structure for calculated DemandProfile data for each shecule.
    
    :var schedule: Schedule loaded for DemandProfile data.
    :var activeBaseline: The active baseline of the schedule.
    :var laborDemand: Labor demand calculated for the schedule.
    :var capitalDemand: Capital Demand calculated for the schedule.
    :var expenseDemand: Expense Demand calculated for the schedule.
    :var currency: Currency used in the calculations (not currently used).
    """
    schedule: Schedule = None
    activeBaseline: Schedule = None
    laborDemand: List[ItemDemand] = ()
    capitalDemand: List[ItemDemand] = ()
    expenseDemand: List[ItemDemand] = ()
    currency: str = ''


@dataclass
class DemandProfileResponses(TcBaseObj):
    """
    This represents the DemandProfileResponses for a schedule.
    
    :var response: The Demand Profile Information of one schedule.
    :var data: The ServiceData.
    """
    response: List[DemandProfileResponse] = ()
    data: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.Reports._2007_06.BOMRollup import CreateRollupCalculationTemplateResponse, CreateRollupReportInput, GetRollupTemplateInput, RollupTemplateInput, RollupTemplateResponse, CreateRollupTemplateInput, RollupReportResponse, ReviseRollupReportInput, CreateRollupCalculationTemplateInput, RollupTemplateCalculationResponse
from typing import List
from tcsoa.utils import TcService


class BOMRollupService(TcService):

    @classmethod
    def getRollupTemplateCalculations(cls, input: List[RollupTemplateInput]) -> RollupTemplateCalculationResponse:
        """
        This operation gets a RollupTemplate's calculation templates. Calculation templates define which properties
        will be used as input for calculations such as sum, center of mass, product of inertia, etc.
        
        Use cases:
        - User wants to find all calculation templates associated with a RollupTemplate.
        - User wants to verify a RollupTemplate has calculation templates in order to avoid generating an empty rollup
        report.
        
        """
        return cls.execute_soa_method(
            method_name='getRollupTemplateCalculations',
            library='Internal-Reports',
            service_date='2007_06',
            service_name='BOMRollup',
            params={'input': input},
            response_cls=RollupTemplateCalculationResponse,
        )

    @classmethod
    def getRollupTemplates(cls, input: List[GetRollupTemplateInput]) -> RollupTemplateResponse:
        """
        This operation gets RollupTemplates by using one of three options:
        1.    All RollupTemplates if the input string is empty, null, or "Mode=ALL". This will find all rollup
        templates in the database.
        2.    User preference - If the input string is "Mode=USER_PREFERENCE", this operation will find the user's
        favorite RollupTemplates (defined by the preference ROLL_UP_FAVORITE_TEMPLATES).
        3.    Specific name and scope context - If input the string is "Mode=SPECIFIC:name:scope_context", this
        operation will search the database for the specific rollup template that matches the name and scope_context
        supplied. Name is the name of the template, scope_context depends on the scope - for site scope, it is the site
        ID; for group, it is the group name; for user, it is the user name.
        
        Use cases:
        - User needs to fetch RollupTemplates in order to clone them.
        - User created RollupTemplates in another session, and needs to fetch them in order to create and attach rollup
        calculation templates.
        - User wants to generate rollup reports.
        - User wants to get a RollupTemplate's calculation templates.
        - User wants to revise a rollup report.
        
        """
        return cls.execute_soa_method(
            method_name='getRollupTemplates',
            library='Internal-Reports',
            service_date='2007_06',
            service_name='BOMRollup',
            params={'input': input},
            response_cls=RollupTemplateResponse,
        )

    @classmethod
    def cloneRollupTemplates(cls, input: List[RollupTemplateInput]) -> RollupTemplateResponse:
        """
        This operation makes a copy or copies of existing RollupTemplates.
        
        Use cases:
        User wants to create a RollupTemplate (and calculation templates) similar to one that already exists.
        """
        return cls.execute_soa_method(
            method_name='cloneRollupTemplates',
            library='Internal-Reports',
            service_date='2007_06',
            service_name='BOMRollup',
            params={'input': input},
            response_cls=RollupTemplateResponse,
        )

    @classmethod
    def reviseRollupReports(cls, input: List[ReviseRollupReportInput]) -> RollupReportResponse:
        """
        This operation revises one or more rollup reports. This is used when the user would prefer to overwrite an
        existing report, instead of generating a new report and keeping the old one.
        
        Use cases:
        User has already generated a rollup report and wants to make some changes, as opposed to simply generating a
        new rollup report.
        """
        return cls.execute_soa_method(
            method_name='reviseRollupReports',
            library='Internal-Reports',
            service_date='2007_06',
            service_name='BOMRollup',
            params={'input': input},
            response_cls=RollupReportResponse,
        )

    @classmethod
    def createRollupCalculationTemplates(cls, input: List[CreateRollupCalculationTemplateInput]) -> CreateRollupCalculationTemplateResponse:
        """
        This operation creates rollup calculation templates.
        
        Use cases:
        User invokes 'createRollupCalculationTemplates' to create and attach rollup calculation templates to
        RollupTemplates.
        """
        return cls.execute_soa_method(
            method_name='createRollupCalculationTemplates',
            library='Internal-Reports',
            service_date='2007_06',
            service_name='BOMRollup',
            params={'input': input},
            response_cls=CreateRollupCalculationTemplateResponse,
        )

    @classmethod
    def createRollupTemplates(cls, input: List[CreateRollupTemplateInput]) -> RollupTemplateResponse:
        """
        The 'createRollupTemplates' operation creates one or more RollupTemplates. RollupTemplates are a container for
        one or more rollup calculation templates.
        
        Use cases:
        User wants to create RollupTemplates in order to generate rollup reports.
        """
        return cls.execute_soa_method(
            method_name='createRollupTemplates',
            library='Internal-Reports',
            service_date='2007_06',
            service_name='BOMRollup',
            params={'input': input},
            response_cls=RollupTemplateResponse,
        )

    @classmethod
    def generateRollupReports(cls, input: List[CreateRollupReportInput]) -> RollupReportResponse:
        """
        This operation creates rollup reports for the specified root BOM lines, using the specified RollupTemplates.
        
        Use cases:
        User has created a RollupTemplate with rollup calculation templates, and wants to generate a rollup report.
        """
        return cls.execute_soa_method(
            method_name='generateRollupReports',
            library='Internal-Reports',
            service_date='2007_06',
            service_name='BOMRollup',
            params={'input': input},
            response_cls=RollupReportResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindCheckedOutsInStructureResponse(TcBaseObj):
    """
    Return structure for findCheckedOutsInStructure operation
    
    :var checkedOutList: This is the structure contains the Tags of all the checked outs objects.
    :var serviceData: This is a common data strucuture used to return sets of Teamcenter
    Data Model object from a service request. This also holds services exceptions.
    """
    checkedOutList: List[BusinessObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class LicenseStatus(TcBaseObj):
    """
    The license status.
    
    :var numPurchasedLicenses: The number of licenses purchased for base license level.
    :var numUsedLicenses: The number of licenses used for base license level.
    :var numPurchasedLicenseBundles: The number of licenses purchased for the license bundle.
    :var numUsedLicenseBundles: The number of licenses used for license bundle.
    """
    numPurchasedLicenses: int = 0
    numUsedLicenses: int = 0
    numPurchasedLicenseBundles: int = 0
    numUsedLicenseBundles: int = 0


@dataclass
class LicenseStatusResponse(TcBaseObj):
    """
    The return data for the activateUsers2 service operation.
    
    :var licStatus: List of LicenseStatus objects, one for each ActivateUserInput object.
    :var serviceData: Object with all activated User objects in updated list and any errors that occurred during
    activating users.
    """
    licStatus: List[LicenseStatus] = ()
    serviceData: ServiceData = None


@dataclass
class ActivateUserInput(TcBaseObj):
    """
    The input arguments for the activateUsers2 service operation.
    
    :var licenseLevel: The license level to be granted to the user.
    :var licenseBundle: The license bundle to be set for the user.
    :var targetUser: The user to be activated and granted the license level.
    """
    licenseLevel: int = 0
    licenseBundle: str = ''
    targetUser: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ReportDefinition
from tcsoa.gen.StructureManagement._2010_09.StructureVerification import EndItemDetail, ReportCriteria
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetAssignmentComparisonDetailsResponse(TcBaseObj):
    """
    response of method getAssignmentComparisonDetails - a vector of AssignmentTypeDetail element and serviceData
    
    :var details: a list of AssignmentType detail elements - for all the input equivalent sets.
    :var serviceData: serviceData element to capture any partial errors.
    """
    details: List[AssignmentTypeDetail] = ()
    serviceData: ServiceData = None


@dataclass
class GetDescendentComparisonDetailsResponse(TcBaseObj):
    """
    structure to capture the response of getDescendentComparisonDetails method. Has the list of details for each input
    set and serviceData to capture partial errors.
    
    :var details: detail list.
    :var serviceData: serviceData member to capture any partial errors.
    """
    details: List[DescendentDetail] = ()
    serviceData: ServiceData = None


@dataclass
class GetPartitionComparisonDetailsResponse(TcBaseObj):
    """
    structure to capture the vector of PartitionDetail elements and serviceData for partial errors.
    
    :var details: vector of partitionDetail elements.
    :var serviceData: serviceData member to capture partial errors.
    """
    details: List[PartitionDetail] = ()
    serviceData: ServiceData = None


@dataclass
class GetPredecessorComparisonDetailsResponse(TcBaseObj):
    """
    return the list of predecessor detail elements and service data for partial errors.
    
    :var details: details about the predecessor detail for each element.
    :var serviceData: serviceData to capture any partial errors.
    """
    details: List[PredecessorDetail] = ()
    serviceData: ServiceData = None


@dataclass
class GetPropertyComparisonDetailsResponse(TcBaseObj):
    """
    response of getPropertyComparisonDetails method.
    
    :var details: the list of elements that have the details for each property in the input.
    :var serviceData: serviceData element to capture any partial errors.
    """
    details: List[PropertyDetail] = ()
    serviceData: ServiceData = None


@dataclass
class AssignmentDetail(TcBaseObj):
    """
    For each line in input equivalent set  holds the tag of the current assigned objects for that line.
    
    :var assignments: size is same as the size of the input equivalent line. It is the list of assignments (one per
    line) - across the row.
    :var matchType: flag to indicate match of the current assignment.
    """
    assignments: List[BusinessObject] = ()
    matchType: int = 0


@dataclass
class AssignmentTypeDetail(TcBaseObj):
    """
    a structure to pair the AssignmentTypeDetail Element with an index into the input vector of equivalent sets of
    objects.
    
    :var index: index into the input vector.
    :var details: the size of the vector is same as maximum number of  assignmentTypes among the set equivalent lines.
    eg: if src1, target1 are equivalent and src1 has 3 manual assignments and target1 has 2 - this vector will have 3
    elements. The first AssignmentTypeDetailElement will have src1_assign1, target1_assign1; the second
    AssignmentTypeDetailElement will have src1_assign2,target1_assign2; the third AssignmentTypeDetailElement will have
    src1_assign3, NULL
    :var equivalentLines: the set of all equivalent lines in input (all equivalent srcs in sequence and then all
    targets in sequence).
    """
    index: int = 0
    details: List[AssignmentTypeDetailElement] = ()
    equivalentLines: List[BusinessObject] = ()


@dataclass
class AssignmentTypeDetailElement(TcBaseObj):
    """
    a structure to capture the details for assignment comparison.
    
    :var assignmentType: a string to indicate the type of assignment (MEConsumed, MEResource etc.)
    :var manualAssignments: length of the vector will be the maximum number of assignments among all the equivalent
    lines for the given type. If there are 3 assigments for src1 and 1 for target1 - the size of this vector will be 3,
    with the first AssignmentDetails element having {src1assign1,target1assign1}, the second AssignmentDetail element
    having {src1assign2,NULLTAG}, the third {src1assign3,NULLTAG}.
    :var logicalAssignments: length of the vector will be the maximum number of logical assignments among all the
    equivalent lines for the given type.
    """
    assignmentType: str = ''
    manualAssignments: List[AssignmentDetail] = ()
    logicalAssignments: List[LogicalAssignmentDetail] = ()


@dataclass
class AsyncACInput(TcBaseObj):
    """
    provides a set of input values for the accountabilityCheckAsync operation.
    
    :var sourceObjects: persistent objects that can be converted to bomlines. Currently, the only supported object is
    VisStructureContext object, with the array length being 1.
    :var targetObjects: persistent objects (target) that can be converted to bomlines. Currently, the only supported
    object is VisStructureContext object, with the array length being 1.
    :var resultName: Name of occurrenceGroup to be created - when report is generated.
    :var resultDesc: optional description of the OccurrenceGroup to be created.
    :var reportCriteria: criteria for printable report.
    :var reportMode: Indicates which mode the accountability check report has to be generated.10 - batch report, 16 -
    batch propagate.
    :var partialMatchCriteria: the set of options to be used for comparison on equivalent lines. It is a map - with the
    key being the name of plugin or string to be used as a discriminator between various components.
    :var includeScopeLines: Flag to indicate whether  to include scope lines as part of result set.
    :var options: mask of integer values representing different UI options.
    :var sourceContextLine: Optional source context line. Currently unused.
    :var targetContextLine: optional target context line. Currently, not used.
    :var matchType: Represents user choice of color display.
    :var sourceFilteringRule: The source filtering rule.
    :var targetFilteringRule: The target filtering rule.
    :var sourceDepth: the depth of source structure. -1 represents all depths.
    :var targetDepth: The depth of target structure from each target root. -1 to set it to any depth.
    """
    sourceObjects: List[BusinessObject] = ()
    targetObjects: List[BusinessObject] = ()
    resultName: str = ''
    resultDesc: str = ''
    reportCriteria: AsyncReportCriteria = None
    reportMode: int = 0
    partialMatchCriteria: AsyncStringToPartialMatchCriteriaMap = None
    includeScopeLines: bool = False
    options: int = 0
    sourceContextLine: BusinessObject = None
    targetContextLine: BusinessObject = None
    matchType: int = 0
    sourceFilteringRule: str = ''
    targetFilteringRule: str = ''
    sourceDepth: int = 0
    targetDepth: int = 0


@dataclass
class AsyncDetails(TcBaseObj):
    """
    details that are to be specified if an operation is to be performed asynchronously. This is same as the BatchUtils
    parameters that is (optionally) passed to accountabilityCheck. Duplicated as structures are not to be shared in soa
    framework.
    
    :var identifier: any user defined string for recognizing the request
    :var mode: processing mode on server. Possible values are "BackGround", "Blocking" and "InProcess" (case
    sensitive). Currently, the only supported value is BackGround. In this mode the Dispatcher services must be
    installed, or the server will default to InProcess (meaning same tcserver as the one the client connects to will be
    used for accountability).
    :var site: processing site. 0 - local. This information is used in the blocking mode to get the http url.
    :var priority: possible values - 0-3, 0 being the lowest.
    :var startTime: start date/time of scheduled dispatcher request
    :var endTime: end date/time of scheduled dispatcher request
    :var daysOfWeek: on which day of the week translator (async process) has to be run. Should have 7 entries and a
    true indicates should be run on that day. Starting on Sunday (1st entry).
    :var endAfterOccurrences: number of times the async process has to run.
    :var primaryObjects: objects used directly or indirectly for the asynchronous processing.
    :var secondaryObjects: any auxiliary objects to be used as additional info during processing of asynchronous
    request. Example - a folder to add some datasets to.
    """
    identifier: str = ''
    mode: str = ''
    site: int = 0
    priority: int = 0
    startTime: datetime = None
    endTime: datetime = None
    daysOfWeek: List[bool] = ()
    endAfterOccurrences: int = 0
    primaryObjects: List[BusinessObject] = ()
    secondaryObjects: List[BusinessObject] = ()


@dataclass
class AsyncPartialMatchCriteria(TcBaseObj):
    """
    a structure to capture generic Partial Match criteria
    
    :var intMap: map of string to vector or integers.
    :var dblMap: map of string to vector of doubles.
    :var strMap: map of string to vector of strings.
    :var objMap: map of string to vector of objects.
    """
    intMap: StringToIntVectorMap = None
    dblMap: StringToDblVectorMap = None
    strMap: StringToStrVectorMap = None
    objMap: StringToObjVectorMap = None


@dataclass
class AsyncReportCriteria(TcBaseObj):
    """
    criteria for generating print report
    
    :var rdTag: report definition Tag (required)
    :var reportName: name of the report.
    :var datasetType: Dataset type to be used.
    :var reportOptionsNames: StructElement name="reportOptionsNames" description="a vector of strings containing the
    Names in a series of Name/Value pairs used to specify additional criteria (optional)
    :var reportOptionsValues: a vector of strings containing the Values in a series of Name/Value pairs used to specify
    additional criteria (optional)
    :var contextObjects: a vector of Tags representing context object(s) (required for item reports)
    :var contextObjectUIDs: a vector of uids representing context objects
    :var stylesheetTag: stylesheet Tag (optional)
    :var stylesheetName: Name of the stylesheet.
    :var datasetName: name of containing DataSet (optional)
    :var criteriaName: a vector of strings containing the Names in a series of Name/Value pairs used to specify
    additional criteria (optional)
    :var criteriaValues: a vector of strings containing the Values in a series of Name/Value pairs used to specify
    additional criteria (optional)
    :var datasetCtxUID: The uid for the context dataset
    :var datasetCtxObj: Dataset context tag
    :var relationName: relation name to be used.
    """
    rdTag: ReportDefinition = None
    reportName: str = ''
    datasetType: str = ''
    reportOptionsNames: List[str] = ()
    reportOptionsValues: List[str] = ()
    contextObjects: List[BusinessObject] = ()
    contextObjectUIDs: List[str] = ()
    stylesheetTag: BusinessObject = None
    stylesheetName: str = ''
    datasetName: str = ''
    criteriaName: List[str] = ()
    criteriaValues: List[str] = ()
    datasetCtxUID: str = ''
    datasetCtxObj: BusinessObject = None
    relationName: str = ''


@dataclass
class LogicalAssignmentDetail(TcBaseObj):
    """
    For each line in input equivalent set  holds the tag of the current assigned objects for that line.
    
    :var criteria: for each line in input equivalent set holds the criteria of the current line's LA - across the row.
    :var resolvedAssignments: length of the vector will be the maximum number of resolved assignments among all the
    equivalent lines for the given type.
    :var matchType: flag to indicate match of the current assignment.
    :var logicalAssignments: For each line in input equivalent set holds the logical assignment or tool requirement
    object.
    """
    criteria: List[str] = ()
    resolvedAssignments: List[AssignmentDetail] = ()
    matchType: int = 0
    logicalAssignments: List[BusinessObject] = ()


@dataclass
class PartialMatchCriteria(TcBaseObj):
    """
    a structure to capture generic Partial Match criteria
    
    :var intMap: map of string to vector or integers.
    :var dblMap: map of string to vector of doubles.
    :var strMap: map of string to vector of strings.
    :var objMap: map of string to vector of objects.
    :var dateMap: map of string to vector of dates
    """
    intMap: StringToIntVectorMap = None
    dblMap: StringToDblVectorMap = None
    strMap: StringToStrVectorMap = None
    objMap: StringToObjVectorMap = None
    dateMap: StringToDateVectorMap = None


@dataclass
class PartitionDetail(TcBaseObj):
    """
    structure to capture the position(index) in the input equivalence set and the partition elements vector.
    
    :var index: position in the input vector.
    :var partitions: The size of the vector matches the equivalent lines. This list the partition per input equivalent
    line as a row.
    :var isDifferent: flag to indicate whether any of the partitions in a single row is different.
    :var equivalentLines: the set of all equivalent lines in input (all equivalent srcs in sequence and then all
    targets in sequence).
    """
    index: int = 0
    partitions: List[BusinessObject] = ()
    isDifferent: bool = False
    equivalentLines: List[BusinessObject] = ()


@dataclass
class PredecessorDetail(TcBaseObj):
    """
    the list of PredecessorDetailElements for each set of input equivalent objects.
    
    :var index: position in the input vector.
    :var details: The size of the vector matches the maximum number of predecessors for each line in input equivalent
    set at index above (given now by equivalentLines - src lines being followed by targetlines). The
    PredecessorDetailElement will be the size of the input lines. Eg: equivalentset(src1,target1) - src1 has 3
    predecessors, target1 has 1 predecessor. details element vector size will be 3. The first PredecessorDetailElement
    will contain 2 predecessors: SrcPred1,targetPred1. The second PredecessorDetailElement will have SrcPred2,NULLTAG,
    and 3rd PredecessorDetailElement will have SrcPred3,NULLTAG.
    :var equivalentLines: the set of all equivalent lines in input (all equivalent srcs in sequence and then all
    targets in sequence).
    """
    index: int = 0
    details: List[PredecessorDetailElement] = ()
    equivalentLines: List[BusinessObject] = ()


@dataclass
class PredecessorDetailElement(TcBaseObj):
    """
    the list of PredecessorDetailElements for each set of input equivalent objects.
    
    :var predecessors: The size of the vector will match the size of the equivalent lines at input index i. This array
    represents one predecessor per input line (row in a table, and not a column). . Eg: SrcPred1, NULLTAG, TargetPred1,
    NULLTAG (assuming 4 eqv. lines)
    :var matchType: 0 means fullmatch, 1 means match, 2 means multiple match
    """
    predecessors: List[BusinessObject] = ()
    matchType: int = 0


@dataclass
class PropagationInput(TcBaseObj):
    """
    the input structure for propagateProperties.
    
    :var target: the target object to which the changes will be propagated.
    :var sources: the source object(s) from which the changes will be propagated to target.
    :var criteria: the partialMatchCriteria structure to be used for change propagation. This is a map of a string (a
    client id) to that client's properties/criteria.
    """
    target: BusinessObject = None
    sources: List[BusinessObject] = ()
    criteria: StringToPartialMatchCriteriaMap = None


@dataclass
class PropagationResponse(TcBaseObj):
    """
    The response of Propagation Service.
    
    :var results: a vector of propagationResults.
    :var logFileTicket: a fms ticket for the transient file that captures the log of propagation service.
    :var serviceData: Service Data object
    """
    results: List[PropagationResult] = ()
    logFileTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class PropagationResult(TcBaseObj):
    """
    a structure to capture a single result for an object that is being propagated from one context to another.
    
    :var index: a integer representing the position in input vector.
    :var propagationResults: a map of string to vector of propagationResultElements. The key of the map would indicate
    the client id corresponding to what is passed in PropagationInput.
    """
    index: int = 0
    propagationResults: PropagationResultMap = None


@dataclass
class PropagationResultElement(TcBaseObj):
    """
    a structure to capture a single result for an object that is being propagated from one context to another.
    
    :var object: an object which is participating in the propagation.
    :var propagatedProperties: successfully propagatedProperties.
    :var logFileTicket: an optional log file ticket. If the extension/server generates a specific logfile - this will
    be the fms ticket for that file (transient).
    """
    object: BusinessObject = None
    propagatedProperties: List[str] = ()
    logFileTicket: str = ''


@dataclass
class PropertyDetail(TcBaseObj):
    """
    the  propertyDetailElement for each of the properties per input vector element.
    
    :var index: index of the input vector of equivalent set of obejcts.
    :var details: the list of PropertyDetailsElement
    """
    index: int = 0
    details: List[PropertyDetailsElement] = ()


@dataclass
class PropertyDetailsElement(TcBaseObj):
    """
    a element to capture the details of a property for an object(s).
    
    :var propertyName: the name of the property.
    :var isDifferent: flag to indicate if any of the objects have a value(s) that is different.
    """
    propertyName: str = ''
    isDifferent: bool = False


@dataclass
class BOMLineNetEffectivityDetail(TcBaseObj):
    """
    structure to encapsulate a BOMLine and the associated endItem effectivity details.
    
    :var line: The bomline for which the effectivity details will added.
    :var effDetails: The array of effectivity details for the bomline
    """
    line: BusinessObject = None
    effDetails: List[EndItemDetail] = ()


@dataclass
class BatchDetails(TcBaseObj):
    """
    details that are to be specified if an operation is to be performed asynchronously.
    
    :var identifier: any user defined string for recognizing the request
    :var mode: processing mode on server. Possible values are "BackGround", "Blocking" and "InProcess" (case
    sensitive). Currently, the only supported value is BackGround. In this mode the Dispatcher services must be
    installed, or the server will default to InProcess (meaning same tcserver as the one the client connects to will be
    used for accountability).
    :var site: processing site. 0 - local. This information is used in the blocking mode to get the http url.
    :var priority: possible values - 0-3, 0 being the lowest.
    :var startTime: start date/time of scheduled dispatcher request
    :var endTime: end date/time of scheduled dispatcher request
    :var daysOfWeek: on which day of the week translator (async process) has to be run. Should have 7 entries and a
    true indicates should be run on that day. Starting on Sunday (1st entry).
    :var endAfterOccurrences: number of times the async process has to run.
    :var primaryObjects: objects used directly or indirectly for the asynchronous processing.
    :var secondaryObjects: any auxiliary objects to be used as additional info during processing of asynchronous
    request. Example - a folder to add some datasets to.
    """
    identifier: str = ''
    mode: str = ''
    site: int = 0
    priority: int = 0
    startTime: datetime = None
    endTime: datetime = None
    daysOfWeek: List[bool] = ()
    endAfterOccurrences: int = 0
    primaryObjects: List[BusinessObject] = ()
    secondaryObjects: List[BusinessObject] = ()


@dataclass
class UnitAndLineDetails(TcBaseObj):
    """
    Structure to capture the Equivalent BomLines (potentially from 2 windows - a source and target window) along with
    their matched units or dates.
    
    :var units: all or a subset of Unit numbers/dates associated with the source and targetlines. The units are in
    pairs - meaning - if you have unit effectivtiy like:1-7,10 - this array will be 1,7,10,10.
    :var dates: If date effectivity is  used - this will have the date ranges for the equivalent lines. Current
    implementation does not support thi
    :var srcLines: list of of srclines that satisfy the units/dates in this structure.
    :var targetLines: list of of target lines ( lines from another window  that are equivalent in some way to the src
    lines - eg: ID in context) that satisfy the units/dates in this structure.
    """
    units: List[int] = ()
    dates: List[datetime] = ()
    srcLines: List[BusinessObject] = ()
    targetLines: List[BusinessObject] = ()


@dataclass
class ACInput(TcBaseObj):
    """
    provides a set of input values for the accountabilityCheck operation.
    
    :var sourceObjects: The source bom lines.
    :var targetObjects: The target bom lines.
    :var resultName: Name of occurrenceGroup to be created - when report is generated.
    :var resultDesc: optional description of the OccurrenceGroup to be created.
    :var reportCriteria: criteria for printable report.
    :var reportMode: Indicates which mode the accountability check report has to be generated. Occurrence group mode of
    excel report mode or coloring mode or a combination of those or batch report or batch propagate. This is a bit
    masked flag. 1 - coloring mode, 2 - generate report, 3 - color and generate report, 4 - occurrence group report, 8
    - batch report, 16 - batch propagate.
    :var partialMatchCriteria: the set of options to be used for comparison on equivalent lines. It is a map - with the
    key being the name of plugin or string to be used as a discriminator between various components.
    :var includeScopeLines: Flag to indicate whether  to include scope lines as part of result set.
    :var options: mask of integer values representing different UI options.
    :var sourceContextLine: Optional source context line.
    :var targetContextLine: optional target context line.
    :var matchType: Represents user choice of color display.
    :var sourceFilteringRule: The source filtering rule.
    :var targetFilteringRule: The target filtering rule.
    :var sourceDepth: the depth of source structure. -1 represents all depths.
    :var targetDepth: The depth of target structure from each target root. -1 to set it to any depth.
    """
    sourceObjects: List[BusinessObject] = ()
    targetObjects: List[BusinessObject] = ()
    resultName: str = ''
    resultDesc: str = ''
    reportCriteria: ReportCriteria = None
    reportMode: int = 0
    partialMatchCriteria: StringToPartialMatchCriteriaMap = None
    includeScopeLines: bool = False
    options: int = 0
    sourceContextLine: BusinessObject = None
    targetContextLine: BusinessObject = None
    matchType: int = 0
    sourceFilteringRule: str = ''
    targetFilteringRule: str = ''
    sourceDepth: int = 0
    targetDepth: int = 0


@dataclass
class CompareNetEffectivityGroup(TcBaseObj):
    """
    structure to capture the response of CompareNetEffectivity method.
    
    :var srcLineEffectivities: effectivities of the source lines.
    :var targetLineEffectivities: effectivities of targetLines.
    :var missingSrcDetails: details of missing src by effectivity comparison.
    :var missingTargetDetails: Details of missing target effectivities
    :var overlappingEffectivities: Details of overlapping effectivities. Source lines that have overlapping units or
    target lines that have overlapping units will be listed.
    :var matchingEffectivities: Details of matching source and target effectivities
    :var isMisMatch: flag that is set to true if there is a mismatch.
    """
    srcLineEffectivities: List[BOMLineNetEffectivityDetail] = ()
    targetLineEffectivities: List[BOMLineNetEffectivityDetail] = ()
    missingSrcDetails: List[EndItemAndUnitDetails] = ()
    missingTargetDetails: List[EndItemAndUnitDetails] = ()
    overlappingEffectivities: List[EndItemAndUnitDetails] = ()
    matchingEffectivities: List[EndItemAndUnitDetails] = ()
    isMisMatch: bool = False


@dataclass
class CompareNetEffectivityResponse(TcBaseObj):
    """
    structure to capture the response of compareNetEffectivityGroup method.  Vector of CompareNetEffectivityGroup
    structures one per input set based equivalent lines and a serviceData member to report partial errors.
    
    :var compareGroups: vector of the compare results for each corresponding set of input vector of equivalent lines
    :var serviceData: serviceData to return any partial errors
    """
    compareGroups: List[CompareNetEffectivityGroup] = ()
    serviceData: ServiceData = None


@dataclass
class DescendentDetail(TcBaseObj):
    """
    the list of DescendentDetailElements for each set of input equivalent objects.
    
    :var index: position in the input vector.
    :var details: The size of the vector matches the maximum number of descendents for each line in input equivalent
    set at index above (given now by equivalentLines - src lines being followed by targetlines). Eg:
    equivalentset(src1,target1) - src1 has 3 children, target1 has 1 child. details element vector size will be 3. The
    first DescendentDetailElement will contain 2 children: SrcChild1,targetChild1. The second DescendentDetailElement
    will have SrcChild2,NULLTAG, and 3rd DescendentDetailElement will have SrcChild3,NULLTAG.
    :var equivalentLines: the set of all equivalent lines in input (all equivalent srcs in sequence and then all
    targets in sequence).
    """
    index: int = 0
    details: List[DescendentDetailElement] = ()
    equivalentLines: List[BusinessObject] = ()


@dataclass
class DescendentDetailElement(TcBaseObj):
    """
    the list of DescendentDetailElements for each set of input equivalent objects.
    
    :var children: The size of the vector will match the size of the equivalent lines at input index i. This array
    represents one predecessor per input line (row in a table, and not a column). . Eg: SrcChild1, NULLTAG,
    TargetChild1, NULLTAG (assuming 4 eqv. lines)
    :var matchType: 0 means fullmatch, 1 means match, 2 means multiple match
    """
    children: List[BusinessObject] = ()
    matchType: int = 0


@dataclass
class EndItemAndUnitDetails(TcBaseObj):
    """
    endItem (item used to specify the unit effectivity) tag and the associated UnitAndLineDetails structure.
    
    :var endItem: Teamcenter::BusinessObject
    The endItem object to be associated with the Unit details
    :var details: structure capturing the details of line(s) and effectivities.
    """
    endItem: BusinessObject = None
    details: List[UnitAndLineDetails] = ()


@dataclass
class EquivalentLines(TcBaseObj):
    """
    Lines from a Source Window and a Target Window that are equivalent. For example - having the same ID in Context or
    other criteria.
    
    :var eqvSrcLines: set of source BOMLine objects that are equivalent based on some criteria like ID in context.
    :var eqvTargetLines: set of target BOMLine objects (not the same window as source lines) that are equivalent in a
    manner consistent with the source lines.
    """
    eqvSrcLines: List[BusinessObject] = ()
    eqvTargetLines: List[BusinessObject] = ()


@dataclass
class EquivalentSetElement(TcBaseObj):
    """
    a structure to capture the equivalent src and target lines along with the partial match criteria.
    
    :var eqvSrcLines: the src lines which are equivalent (currently only id in context)
    :var eqvTargetLines: equivalent target lines - based on In context id (absoccid) or apn or ebop criteria (origin
    link, logical designator, uid )
    :var criteria: input map for specifying criteria, the key being server extension id.
    """
    eqvSrcLines: List[BusinessObject] = ()
    eqvTargetLines: List[BusinessObject] = ()
    criteria: StringToPartialMatchCriteriaMap = None


"""
a map of string to AsyncPartialMatchCriteriaMap
"""
AsyncStringToPartialMatchCriteriaMap = Dict[str, AsyncPartialMatchCriteria]


"""
a map of string (an id) and the vector of PropagationResult objects.
"""
PropagationResultMap = Dict[str, List[PropagationResultElement]]


"""
a map of string to vector of dates
"""
StringToDateVectorMap = Dict[str, List[datetime]]


"""
String to vector of doubles map.
"""
StringToDblVectorMap = Dict[str, List[float]]


"""
map of string to vector of integers.
"""
StringToIntVectorMap = Dict[str, List[int]]


"""
a map of string to vector of objects.
"""
StringToObjVectorMap = Dict[str, List[BusinessObject]]


"""
a map of string to PartialMatchCriteriaMap
"""
StringToPartialMatchCriteriaMap = Dict[str, PartialMatchCriteria]


"""
a map of String to vector of strings.
"""
StringToStrVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2012_02.StructureVerification import EquivalentLines, ACInput, BatchDetails
from tcsoa.gen.StructureManagement._2016_05.StructureVerification import AccountabilityCheckResponse
from tcsoa.gen.StructureManagement._2013_05.StructureVerification import AttributeGroupAndFormComparisonResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.utils import TcService


class StructureVerificationService(TcService):

    @classmethod
    def accountabilityCheck2(cls, input: ACInput, batchDetails: BatchDetails) -> AccountabilityCheckResponse:
        """
        The operation will call the existing accountability check functions, which will generate a check result for
        report in the colored display
        """
        return cls.execute_soa_method(
            method_name='accountabilityCheck2',
            library='StructureManagement',
            service_date='2016_05',
            service_name='StructureVerification',
            params={'input': input, 'batchDetails': batchDetails},
            response_cls=AccountabilityCheckResponse,
        )

    @classmethod
    def getAttrGrpsAndFormsComparisonDetail(cls, equivalentObjects: List[EquivalentLines], attributeGroupsNames: List[str], sourceConfigContext: BusinessObject, targetConfigContext: BusinessObject) -> AttributeGroupAndFormComparisonResponse:
        """
        This operation returns the details of differences between the supplied Attribute Groups for the supplied
        equivalent objects (that can be Cpd0DesignElement, Cpd0DesignFeature, or BOMLine objects) and the supplied
        configuration contexts for source equivalent object and target equivalent object respectively.For 4gd to 4gd
        compare, attribute group names includes attribute groups and managed attribute groups.Source and target
        configuration context is needed if the attribute group names include Managed attribute group properties.For
        each supplied attribute group the operation returns the list of its attributes, the attributes values for each
        supplied source and target, and the result of comparing each attribute on all supplied sources and targets.
        """
        return cls.execute_soa_method(
            method_name='getAttrGrpsAndFormsComparisonDetail',
            library='StructureManagement',
            service_date='2016_05',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects, 'attributeGroupsNames': attributeGroupsNames, 'sourceConfigContext': sourceConfigContext, 'targetConfigContext': targetConfigContext},
            response_cls=AttributeGroupAndFormComparisonResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision, RevisionRule, BOMLine
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetConfiguredItemRevisionInfo(TcBaseObj):
    """
    Contains the item/item revision object reference and revision rule object reference to find the configured
    itemRevision.
    
    :var object: object reference of the item / item revision
    :var revRule: Teamcenter::RevisionRuleImpl object reference
    """
    object: BusinessObject = None
    revRule: RevisionRule = None


@dataclass
class GetConfiguredItemRevisionOutput(TcBaseObj):
    """
    The response for the getConfiguredItemRevision operation.
    
    :var inputInfo: Member of type GetConfiguredItemRevisionInfo, copy of input data
    :var configuredItemRev: ItemRevision object reference found
    """
    inputInfo: GetConfiguredItemRevisionInfo = None
    configuredItemRev: ItemRevision = None


@dataclass
class GetConfiguredItemRevisionResponse(TcBaseObj):
    """
    The response for the getConfiguredItemRevision call.
    
    :var output: A List of type GetConfiguredItemRevisionOutput
    :var serviceData: The SOA framework object containing plain objects and error information.
    """
    output: List[GetConfiguredItemRevisionOutput] = ()
    serviceData: ServiceData = None


@dataclass
class VariantCondInfo(TcBaseObj):
    """
    Contains the information neede to create/update a variant condition.
    
    :var optionName: optionName
    :var joinOperator: joinOperator  Legal values are: 'AND', 'OR', 'OPEN_BRACKET' and 'CLOSE_BRACKET'.
    :var compOperator: compOperator. Legal values are: 'EQ', 'NEQ', 'LT', 'GT', 'GTEQ' and 'LTEQ'.
    :var value: value
    """
    optionName: str = ''
    joinOperator: VariantOperator = None
    compOperator: ComparisonOperator = None
    value: str = ''


@dataclass
class ClassicOptionInfo(TcBaseObj):
    """
    Contains the information about classic variant option.
    
    :var optionName: Refers to classic variant option name.
    :var optionDesc: Refers to classic variant option description.
    :var values: Refers to list of classic variant option values. This input is given for new or update scenarios.
    :var existingValues: Refers to list of existing classic variant option values.
    """
    optionName: str = ''
    optionDesc: str = ''
    values: List[str] = ()
    existingValues: List[str] = ()


@dataclass
class CreateOrUpdateVariantCondInput(TcBaseObj):
    """
    Contains the input for create/update variant condition associated with a BOMLine object.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var operation: This indicates the operation type which can be create or update variant condition. Legal values
    are: 'Create' and 'Update'.
    :var bomLine: Refers to BOMLine object on which variant condition has been defined.
    :var variantCondInfo: Refers to a list of 'VariantCondInfo' struct, and contains the information needed to
    create/update a variant condition.
    """
    clientId: str = ''
    operation: Operation = None
    bomLine: BOMLine = None
    variantCondInfo: List[VariantCondInfo] = ()


@dataclass
class CreateUpdateClassicOptionsInput(TcBaseObj):
    """
    Contains the input for creating or updating classic variant options associated with a BOMLine object.
    
    :var clientId: Refers to clientId attribute.
    :var operation: Refers to operation type, as defined in 'OptionOperation'. The operation could be 'CreateOption',
    'AddValue', 'RemoveValue' or 'ReplaceValue'.
    :var bomLine: Refers to BOMLine object on which classic variant options are created or updated.
    :var options: Refers to list of 'ClassicOptionInfo' structure which needs to be updated.
    """
    clientId: str = ''
    operation: OptionOperation = None
    bomLine: BOMLine = None
    options: List[ClassicOptionInfo] = ()


@dataclass
class DelClassicOptionsInput(TcBaseObj):
    """
    Contains the input for deleting variant condition associated with a BOMLine object.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var bomLine: Refers to BOMLine object on which classic variant options are defined, which needs to be deleted.
    :var optionNames: Refers to list of classic variant option names which are needs to be deleted.
    """
    clientId: str = ''
    bomLine: BOMLine = None
    optionNames: List[str] = ()


@dataclass
class DeleteVariantCondInput(TcBaseObj):
    """
    This contains the input for deleting variant condition associated with a BOMLine object.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var bomLine: Refers to BOMLine object on which variant conditions are defined, which needs to be deleted.
    """
    clientId: str = ''
    bomLine: BOMLine = None


class ComparisonOperator(Enum):
    """
    Contains comparison operator. Legal values are: 'EQ, NEQ, LT, GT, GTEQ' and 'LTEQ'.
    """
    EQ = 'EQ'
    NEQ = 'NEQ'
    LT = 'LT'
    GT = 'GT'
    GTEQ = 'GTEQ'
    LTEQ = 'LTEQ'


class Operation(Enum):
    """
    Contains the operation type. Legal values are: 'Create' and 'Update'.
    """
    Create = 'Create'
    Update = 'Update'


class OptionOperation(Enum):
    """
    The operations on variant option. Legal values 'are: CreateOption' ,'AddValue' ,'RemoveValue'  and 'ReplaceValue'.
    """
    CreateOption = 'CreateOption'
    AddValue = 'AddValue'
    RemoveValue = 'RemoveValue'
    ReplaceValue = 'ReplaceValue'


class VariantOperator(Enum):
    """
    Contains the join operator type. Legal values are: 'AND, OR '.
    """
    AND = 'AND'
    OR = 'OR'



from __future__ import annotations

from tcsoa.gen.Classification._2007_01.Classification import TypedDocument, AttributeFormat
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtendedClassDef(TcBaseObj):
    """
    This structure contains the extended information about a class.
    
    :var classDef: This basic information about the class.
    :var parentDefs: List of the parent definitions specific to the class and the parent corresponding basic properties.
    :var classShortName: The short name of the class.
    :var classDescription: The description of the class.
    :var classUserData1: User data 1 added to the class.
    :var classUserData2: User data 2 added to the class.
    :var classChildCount: The count of child classes of the class. If not retrieved, the count is -1.
    :var classInstanceCount: The count of instances in the class. If not retrieved, the count is -1.
    :var classImagesLoaded: If true, value indicates that the image documents for this class were loaded. False, they
    were not loaded.
    :var classImageDocuments: A list of class documents attached to the class (like class image).
    """
    classDef: BaseClassDef = None
    parentDefs: List[BaseClassDef] = ()
    classShortName: str = ''
    classDescription: str = ''
    classUserData1: str = ''
    classUserData2: str = ''
    classChildCount: int = 0
    classInstanceCount: int = 0
    classImagesLoaded: bool = False
    classImageDocuments: List[TypedDocument] = ()


@dataclass
class GetChildrenExtendedOutput(TcBaseObj):
    """
    Structure containing the found classes for the given search criteria. Any failures with the search will be mapped
    to the error message in the ServiceData list of partial errors.
    
    :var classDescriptions: List of child classes found.
    """
    classDescriptions: List[ExtendedClassDef] = ()


@dataclass
class GetChildrenExtendedResponse(TcBaseObj):
    """
    Contains a list of child classes found for the given classes.
    
    :var classDescriptions: List of child classes found for input class IDs.
    :var svcData: Any failures with class ID mapped to the error message are returned in the ServiceData list of
    partial errors.
    """
    classDescriptions: List[GetChildrenExtendedOutput] = ()
    svcData: ServiceData = None


@dataclass
class BaseClassDef(TcBaseObj):
    """
    This structure contains the basic information about a class.
    
    :var classID: The ID of the class.
    :var classBO: Reference to the business object representing this class.
    :var className: The name of the class.
    :var isAbstract: If true, this class is abstract. Classification instances cannot be stored in this class. False,
    indicates that Classification instances can be stored in this class.
    :var classUnitBase: The unit system of the class:
    - 0 - The class supports the storage of metric objects only 
    - 1 - The class supports the storage of non-metric only 
    - 2 - The class supports both metric and non-metric objects
    
    
    :var classIconLoaded: If true, the icon file for this class is loaded. False, it is not loaded.
    :var classIconDocument: The icon document attached to this class.
    """
    classID: str = ''
    classBO: BusinessObject = None
    className: str = ''
    isAbstract: bool = False
    classUnitBase: int = 0
    classIconLoaded: bool = False
    classIconDocument: TypedDocument = None


@dataclass
class SearchClassesExtendedOutput(TcBaseObj):
    """
    Structure containing the found classes for the given search criteria. Any failures with the search will be mapped
    to the error message in the ServiceData list of partial errors.
    
    :var classDescriptions: List of classes found for input search criteria.
    """
    classDescriptions: List[ExtendedClassDef] = ()


@dataclass
class SearchClassesExtendedResponse(TcBaseObj):
    """
    Structure containing the found classes for the given search criteria. Any failures with the search will be mapped
    to the error message in the ServiceData list of partial errors.
    
    :var classDescriptions: List of classes found for input search criteria.
    :var svcData: Any failures with the search will be mapped to the error message in the ServiceData list of partial
    errors.
    """
    classDescriptions: List[SearchClassesExtendedOutput] = ()
    svcData: ServiceData = None


@dataclass
class ValueConversionInput(TcBaseObj):
    """
    Structure containing the value(s) and unit systems which should be used to convert the input values.
    
    :var inputValues: The input values. For regular attributes it's just one value. In case of VLA (variable length
    array) attributes each value has its own entry.
    The calling client is responsible for converting the different property types (int, float, date .etc) to a string
    using the appropriate functions in the SOA client framework's Property class.
    :var inputUnit: The input unit e.g. the unit definition ID "Length_mm" or also the display name "mm".
    :var outputUnit: The output unit e.g. the unit definition ID "Length_mm" or also the display name "mm".
    :var outputFormat: The output format to which the converted value should be formatted.
    :var options: The options parameter (This option is for future use).
    """
    inputValues: List[str] = ()
    inputUnit: str = ''
    outputUnit: str = ''
    outputFormat: AttributeFormat = None
    options: int = 0


@dataclass
class ValueConversionOutput(TcBaseObj):
    """
    The converted values. For regular attributes it's just one value, in case of VLA (variable length array)
    attributes, each value has its own entry.
    
    :var convertedValues: The converted values. For regular attributes it's just one value. In case of VLA (variable
    length array) attributes each value has its own entry.
    
    The calling client is responsible for converting the string value to the different property types  (int, float,
    date .etc) using the appropriate functions in the SOA client framework's Property class.
    """
    convertedValues: List[str] = ()


@dataclass
class ConvertValuesResponse(TcBaseObj):
    """
    Structure containing the converted values. Any failures with the conversion will be mapped to the error message in
    the ServiceData list of partial errors.
    
    :var convertedValues: The converted values.
    :var svcData: Any failures with the conversion will be mapped to the error message in the ServiceData list of
    partial errors.
    """
    convertedValues: List[ValueConversionOutput] = ()
    svcData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import POM_object, BusinessObject, RevisionRule, BOMLine, Folder, TC_Project, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CloneStructureDefaultNaming(TcBaseObj):
    """
    The CloneStructureDefaultNaming structure conatians the information to help form new Item IDs.
    
    :var autogen: Auto assign new item IDs. If set to True this takes precedence over all other parameters.
    :var prefix: Substring to be prefixed to the old item ID to form a new item ID. This parameter can be used in
    conjuction with the other parameters except when autogen is set to True.
    :var suffix: Substring to be suffixed to the old item ID to form a new item ID. This parameter can be used in
    conjuction with the other parameters except when autogen is set to True..
    :var from: Substring to be replaced from the old item ID by another substring to form a new item ID. This parameter
    is required to work in conjuction with the "to" parameter but can be used in conjuction with the other parameters
    except when autogen is set to True.
    :var to: Substring to replace with a substring from the old item ID to form a new item ID. This parameter is
    required to work in conjuction with the "from" parameter but can be used in conjuction with the other parameters
    when except autogen is set to True.
    """
    autogen: bool = False
    prefix: str = ''
    suffix: str = ''
    from_: str = ''
    to: str = ''


@dataclass
class CloneStructureExpandOrUpdateDuplicateItemsOutput(TcBaseObj):
    """
    The CloneStructurExpandOrUpdateDuplicateItemsOutput represents the collection of all related data objects with the
    structure ItemRevision object(s) or part family masters ItemRevision object(s).
    
    :var parentObject: The BOM structure ItemRevision or a part family master ItemRevision.
    :var parentObjectCloneable: Indicates if the object can be cloned. 
    0 - users choice to clone or not
    1 - mandatory - If it is mandatory that cad dependent type will come in pre-checked and the user will not be able
    to de-select it.
    2 - Must not be cloned - If set the cad dependent type will come in unchecked and the user will not be able to
    select it.
    
    :var relatedObjectInfo: List of all dependent objects that are related to the object.
    """
    parentObject: ItemRevision = None
    parentObjectCloneable: int = 0
    relatedObjectInfo: List[CloneStructureRelatedItemsInfo] = ()


@dataclass
class CloneStructureExpandOrUpdateItemsInfo(TcBaseObj):
    """
    The CloneStructureExpandOrUpdateItemsInfo structure contains the information needed to conduct an expand or update
    of a BOM structure that will be duplicated.
    
    :var bomline: If it is not null, expand it and get its dependencies based on the depTypes. The BOMLine can be any
    line in the structure that the user picks for expansion.
    :var itemRevs: The ItemRevision objects to perform search dependencies on. If empty, the ItemRevision objects to be
    used will be from the expansion of the BOMLine.
    :var cadOptions: The dependency types to use for the operation. 
    Following are the CAD options values that are used for CAD Dependency searches: 
    Drawings, Required, PartFamilyMaster, PartFamilyMember, AllDep and  Internal.
    """
    bomline: BOMLine = None
    itemRevs: List[ItemRevision] = ()
    cadOptions: List[str] = ()


@dataclass
class CloneStructureExpandOrUpdateResponse(TcBaseObj):
    """
    The CloneStructureExpandOrUpdateResponse structure contains the results of the structure dependent data search.
    
    :var outputs: List of CloneStructureExpandOrUpdateDuplicateItemsOutput which contains information about the BOM
    structure ItemRevision to its related data relationships.
    :var additionalDependencies: List of available non static CAD dependencies that are derived from the CAD dependency
    closure rules
    :var serviceData: The error information will be recorded here.
    """
    outputs: List[CloneStructureExpandOrUpdateDuplicateItemsOutput] = ()
    additionalDependencies: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class CloneStructureInputInfo(TcBaseObj):
    """
    CloneStructureInputInfo that contains all the information needed to validate and duplicated multiple structures in
    one API call.
    
    :var topLines: The top BOMLine or selected BOMLine of the structure to be cloned. The user can select a sub
    assembly from the original structure to clone. (If running Async mode this input needs to be NULL.)
    :var cadOptions: List of CAD dependency options to process and traverse in the structure to be duplicated.
    Typical options would be Drawings, Required, PartFamilyMaster, PartFamilyMember, AllDep and Internal.
    :var defaultName: The pattern to form the new ItemId.
    :var dataMap: A struct vector of original ItemRevisions and instructions which tells duplicate operation what to do
    with each component of the structure.
    :var cloneFlags: A bitmap for the duplicate flags set.
    1 - smart selection using topLine assigned projects
    2 - rename cad files
    4 - return cloned object map information
          (nor supported in background mode) 
          (RAC will always set this to false)
    8 - run duplicate in background mode
          (this activates the Async mode if it is available)
    16- run duplicate in validate mode
    
    :var projects: The user selected projects to which to add the duplicated Item objects.
    :var defaultFolder: The folder to add the duplicated structures into. (can be NULL)
    :var revRule: If duplicate operation is being run in the background send in the revRule to use when creating the
    new BOMWindow (required for Async).
    :var topItemRevs: If duplicate operation is being run in the background need the top item revision of the structure
    to create the new BOM Window (required for Async).
    """
    topLines: List[BOMLine] = ()
    cadOptions: List[str] = ()
    defaultName: CloneStructureDefaultNaming = None
    dataMap: List[CloneStructureSaveAsIn] = ()
    cloneFlags: int = 0
    projects: CloneStructureProjectInfo = None
    defaultFolder: Folder = None
    revRule: RevisionRule = None
    topItemRevs: List[ItemRevision] = ()


@dataclass
class CloneStructureProjectInfo(TcBaseObj):
    """
    The CloneStructureProjectInfo contains project information for Structure Clone to validate the projects and assign
    the cloned objects to the user selected projects.
    
    :var assign: Flag that determines if projects are to be assigned to new cloned objects.
    :var validate: Flag that determines if validation needs to be executed on projects.
    :var projects: List of projects that the new cloned objects will be assigned to.
    """
    assign: bool = False
    validate: bool = False
    projects: List[TC_Project] = ()


@dataclass
class CloneStructureRelatedItemsInfo(TcBaseObj):
    """
    The CloneStructureRelatedItemsInfo represents the dependent data found and how it is related to a structure
    ItemRevision. RelatedItemsInfo also can represent a family table member to a family table master.
    
    :var relatedObject: An ItemRevision that is the related data.
    :var relatedObjectCloneable: Indicates if the relObj can be cloned.
    0 - users choice to clone or not
    1 - mandatory - If it is mandatory that cad dependent type will come in pre-checked and the user will not be able
    to de-select it.
    2 - Must not be cloned - If set the cad dependent type will come in unchecked and the user will not be able to
    select it
    
    :var relatedObjectGRMName: The GRM relationship used to find the related data.
    :var relatedObjectCADOptionName: The cad dependency type use for the operation.
    """
    relatedObject: ItemRevision = None
    relatedObjectCloneable: int = 0
    relatedObjectGRMName: str = ''
    relatedObjectCADOptionName: str = ''


@dataclass
class CloneStructureResponse(TcBaseObj):
    """
    The CloneStructureResponse structure contains the response information for the cloneStructure API. 
    
    :var dataMap: This is populated when validation of BOM structure fails or validateOnly flag is set to true. Reason
    for returning this information is it allows the calling client to reuse this data to speed the re-validation of the
    structure once the user fixes the error. The struct vector will contain the original Item Revisions, operation
    type, replacing item revision, MFK, PropertyValues and DeepCopyData overrides for each level of the structure.
    
    If validation passes and validateOnly flag is set to false this will be empty.
    
    :var clonedItemRevInfoMap: Map of bomlines to their new cloned item rev and clone status.
    :var clonedObjectInfoMap: Map of original objects and their cloned equivalent (will be empty if validation fails).
    (not support in Async mode)
    :var serviceData: Standard service data return where the partial errors are placed.
    """
    dataMap: List[CloneStructureSaveAsIn] = ()
    clonedItemRevInfoMap: CloneStructureTopLineDataMap = None
    clonedObjectInfoMap: CloneStructureObjectMap = None
    serviceData: ServiceData = None


@dataclass
class CloneStructureSaveAsIn(TcBaseObj):
    """
    The CloneStructureSaveAsIn structure contains all the instructions that tells duplicate what to do with the
    children of the structure to be duplicated.
    
    :var origItemRevComp: Original item revision component to be cloned.
    :var cloneOperationType: Indicates if the original item revision component is to be:
    0 - Clone, component is part of the structure to be cloned and we want to clone the component.
    1 - Reference, component is part of the structure to be cloned and we want to reference, not clone the component.
    2 - Revise, component is part of the structure to be cloned and we want to revise the component.
    3 - Replace, component is part of the structure to be cloned and we want to replace the component with an existing
    part.
    4 - Include, component is not a part of the structure to be cloned and we want to clone the component because it
    has some relation to the structure to be cloned that is not published to Teamcenter.
    
    :var newItemRevinfo: Struct that contains all information related to item revision based on following operations:
    
    - Clone/Revise - will contain Property values (MFK complaint) to support business object "save-as" and "revise"
    operations
    - Reference - will be empty
    - Replace - will contain replacing Item revision "item_id" and "rev_id" values.
    
    
    :var deepCopyDataOverride: Vector of DeepCopyData elements that instructs the "Save As" operation what attached
    data to clone and not clone (Save As Descriptors) for the item revision.
    Note: Reference, Revise and Replace operation will not use this parameter and such it will be empty.
    """
    origItemRevComp: ItemRevision = None
    cloneOperationType: int = 0
    newItemRevinfo: PropertyValues = None
    deepCopyDataOverride: List[DeepCopyData] = ()


@dataclass
class CloneStructureTopLineData(TcBaseObj):
    """
    The CloneStructureTopLineData structure contains the high-level status information of a BOM structure that was
    validated and duplicated.
    
    :var bomline: The top line assembly that represent the original Item revision that was cloned to make the new
    cloned Item revision.
    :var orignalItemRev: Original Item Revision that was cloned to make the new cloned Item Revision.
    :var clonedNewItemRev: New cloned topline item revision.
    :var status: Status of cloning the topline.
    0 - Clone Successful
    46229 - Clone Failed
    46230 - Clone With Partial Errors
    46231 - Validate Failed
    46232 - Clone Asynchronous Job Submitted
    """
    bomline: BOMLine = None
    orignalItemRev: ItemRevision = None
    clonedNewItemRev: ItemRevision = None
    status: int = 0


@dataclass
class DeepCopyData(TcBaseObj):
    """
    The DeepCopyData data structure holds the relevant information regarding applicable deep copy rules
    
    :var attachedObject: Other side object.
    :var propertyName: Name of relation type or reference property for which DeepCopy rule is configured.
    :var propertyType: If Relation, it represents Relation type property. If Reference, it represents Reference property
    :var copyAction: DeepCopy action [NoCopy, CopyAsReference, CopyAsObject, Select ].
    :var isTargetPrimary: If true the target object is processed as primary, otherwise it is processed as a secondary
    object.
    :var isRequired: If true, the copy action can not be modified. If false, the copy action can be changed to
    different action by the user.In this case, the copy action field in the revise dialog is editable.
    :var copyRelations: If true, the custom properties on the source relation object are copied over to the
    newly-created relation. If false, custom properties are not copied.
    :var operationInputTypeName: OperationInput type name.
    :var childDeepCopyData: List of DeepCopy data for the secondary objects on the other side.
    :var operationInputs: OperationInput field to capture property values of attached object, to apply on copied object
    of attached object. Map of property name(key) and property values(values) in string format of attached object, to
    be set on copied object of attached object. The calling client is responsible for converting the different property
    types (int, float, date .etc) to a string using the appropriate toXXXString functions in the SOA client framework
    Property class.
    """
    attachedObject: BusinessObject = None
    propertyName: str = ''
    propertyType: str = ''
    copyAction: str = ''
    isTargetPrimary: bool = False
    isRequired: bool = False
    copyRelations: bool = False
    operationInputTypeName: str = ''
    childDeepCopyData: List[DeepCopyData] = ()
    operationInputs: PropertyValues = None


"""
This map is of property name (as key) and property values (as value) in string format. Each value is a list of strings to support both single valued and multi valued properties of types. The calling client is responsible for converting the different property types (like integer, double, date, etc) to a string using the appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class.


"""
PropertyValues = Dict[str, List[str]]


"""
This maps the original objects  to the new cloned objects
"""
CloneStructureObjectMap = Dict[POM_object, POM_object]


"""
Map that maps original Topline to new cloned Topline with its status
"""
CloneStructureTopLineDataMap = Dict[BOMLine, CloneStructureTopLineData]



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Multisite._2020_12.ImportExportTCXML import MultisiteDashboardResponse, StrVecMap, ReportRecipe
from tcsoa.utils import TcService


class ImportExportTCXMLService(TcService):

    @classmethod
    def getMultisiteDashBoardData(cls, sessionOptions: StrVecMap, reportRecipe: List[ReportRecipe]) -> MultisiteDashboardResponse:
        """
        The getMultisiteDashboard operation provides the historic dashboard data for each site in Multi-Site federation
        which helps to visualize the critical errors occurred during Multi-Site import, export and synchronization
        operations. It also helps to track and analyze the key data points to monitor the health of Multi-Site
        operation.
        
        Use cases:
        In the Multi-Site federation, a user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. During the import/export
        operation errors can occur due to network, data condition etc. These errors should be tracked analyzed and
        displayed in dashboard so that the user can take appropriate actions.
        """
        return cls.execute_soa_method(
            method_name='getMultisiteDashBoardData',
            library='Internal-Multisite',
            service_date='2020_12',
            service_name='ImportExportTCXML',
            params={'sessionOptions': sessionOptions, 'reportRecipe': reportRecipe},
            response_cls=MultisiteDashboardResponse,
        )



from __future__ import annotations

from tcsoa.gen.Allocations._2007_01.Allocation import AllocationContextInput, AllocationLineInput, AllocationWindowInfo, AllocationLineInfo, GetAllocatedBOMViewResponse, GetAllocationWindowResponse
from tcsoa.gen.BusinessObjects import AllocationLine, AllocationWindow, AllocationMapRevision, BusinessObject, RevisionRule, ItemRevision, PSBOMView, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class AllocationService(TcService):

    @classmethod
    def addAllocationLines(cls, allocWindowInput: AllocationWindow, inputAllocationLines: List[AllocationLineInfo]) -> ServiceData:
        """
        The operation is used to create new AllocationLine objects between BOMLine objects of different product
        structures. This operation needs an AllocationWindow object with an associated AllocationMapRevision object,
        for which AllocationLine will be created and a list of structure AllocationLineInfo which consists of name,
        type and reason for creation AllocationLine with from and to BOMLines from different structure. This operation
        will return created AllocationLine objects in the service data as created objects. If BOMLine is same for
        source and target, then error will be reported and stored as service data error.
        
        Use cases:
        AllocationLine is a runtime representation of an Allocation object as well as it provides runtime properties.
        It will maintain sources and targets information for Allocation. When you want traceability between different
        aspects of a product, you can use this operation to create AllocationLine objects.
        """
        return cls.execute_soa_method(
            method_name='addAllocationLines',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput, 'inputAllocationLines': inputAllocationLines},
            response_cls=ServiceData,
        )

    @classmethod
    def closeAllocationWindow(cls, allocWindowInput: AllocationWindow, force: bool) -> ServiceData:
        """
        The operation closes the AllocationWindow and returns the unique identifier string of the AllocationWindow
        business Object in the deleted object list of ServiceData
        
        Use cases:
        Use case 1: Close Allocation Window after create 
        - Create new AllocationWindow, and adding allocations between multiple product structure and save, modify
        further and the AllocationWindow will be closed. Any unsaved Allocation changes will not be saved to Teamcenter
        before close.
        
        
        
        Use case 2: Close Allocation Window after Open
        - Open an AllocationWindow using an existing AllocationMapRevsion object. Add necessary Allocation objects,
        modify any existing Allocation objects, delete any unwanted Allocation. Save the AllocationWindow. Close the
        AllocationWindow. Any unsaved Allocation changes will not be saved to Teamcenter when the AllocationWindow
        closes.
        
        """
        return cls.execute_soa_method(
            method_name='closeAllocationWindow',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput, 'force': force},
            response_cls=ServiceData,
        )

    @classmethod
    def modifyAllocationLines(cls, allocWindowInput: AllocationWindow, inputAllocationLines: List[AllocationLineInput]) -> ServiceData:
        """
        The operation is used to modify AllocationLine objects created between BOMLine objects of different product
        structures which are already present in an AllocationWindow. This operation needs an AllocationWindow object
        with an associated AllocationMapRevision object, for which AllocationLine will be modified for source or target
        BOMLine objects and a list of structure AllocationLineInput which consists of AllocationLine object and
        structure AllocationLineInfo. An AllocationLineInfo object information like from and to BOMLine objects are
        used for this modification purpose. This operation will return the modified AllocationLine objects in the
        service data as updated objects. If the BOMLine is the same for both source and target, then an error will be
        reported and stored as a service data error.
        
        Use cases:
        AllocationLine is a runtime representation of an Allocation object as well as it provides runtime properties.
        It will maintain source and target information for an Allocation. Use this operation to modify AllocationLine
        objects between different aspects of a product.
        """
        return cls.execute_soa_method(
            method_name='modifyAllocationLines',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput, 'inputAllocationLines': inputAllocationLines},
            response_cls=ServiceData,
        )

    @classmethod
    def openAllocationWindow(cls, allocWindowInput: AllocationWindowInfo, icContext: BusinessObject) -> GetAllocationWindowResponse:
        """
        The operation creates an AllocationWindow for the given allocation context object. After creating the
        AllocationWindow object, the operation opens the AllocationWindow and sets the RevisionRule and the incremental
        change context on the opened AllocationWindow. The operation finds all the configured AllocationLine objects
        corresponding to the given BOMWindow objects and returns them to the user along with the AllocationWindow
        object.
        
        Use cases:
        The AllocationWindow is usually created and opened to find different allocations between two or more BOM View
        objects. For example the user wants to find various functions defined in the functional model of a product
        allocated to different ECUs in the logical model. In this case an AllocationWindow for given allocation context
        is created and opened. All the function to ECU allocations relevant to the opened functional and logical model
        for the given revision rule and incremental change context are fetched and displayed to the user in the form of
        AllocationLine objects.
        """
        return cls.execute_soa_method(
            method_name='openAllocationWindow',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput, 'icContext': icContext},
            response_cls=GetAllocationWindowResponse,
        )

    @classmethod
    def saveAllocationWindow(cls, allocWindowInput: AllocationWindow) -> ServiceData:
        """
        Saves any modifications made in the AllocationWindow. Newly created Allocation objects will be saved to
        Teamcenter. Allocation objects which are deleted will be removed from Teamcenter. Any modification made to the
        Allocation will be updated in Teamcenter. If any new Incremental Change objects were created in this
        AllocationWindow, they will be saved to Teamcenter. If any AllocationLine objects are removed and if it is part
        of an Incremental Change object, it will be removed from Teamcenter. All changes made in the AllocationWindow
        will be saved and after save the AllocationLine objects of the AllocationWindow will be returned to the client.
        
        Use cases:
        Use case 1: Save Allocation Window after create and add allocations
        - After creation of a new AllocationWindow new AllocationLine objects can be added. After creating necessary
        allocations, the AllocationWindow is saved to save the changes to Teamcenter.
        
        
        
        Use case 2: Save Allocation Window after open and modification
        - After creation of a new AllocationWindow or opening an existing AllocationWindow, new AllocationLine objects
        can be added, existing allocations can be removed or modified for both source and target. After making
        necessary changes, the AllocationWindow is saved to save the changes to Teamcenter.
        
        """
        return cls.execute_soa_method(
            method_name='saveAllocationWindow',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput},
            response_cls=ServiceData,
        )

    @classmethod
    def createAllocationContext(cls, input: AllocationContextInput) -> GetAllocationWindowResponse:
        """
        This operation is used to create an AllocationMap and AllocationMapRevision object  from the 
        AllocationContextInput structure. AllocationContextInput  consists of Name, id, Revision, type, list  of BOM
        (Bill of Material) Windows for which allocations has to be created. In this operation, it will create an
        AllocationMapRevision and sets it window to said context.
        
        Use cases:
        Allocation allows to map one or more BOMLine objects of two or more product structures. An AllocationMap is a
        business object that specifies how two structures are tied together by a set of allocations that exist between
        two structures. Defining such an AllocationMap needs allocation context which is carried out by this operation.
        """
        return cls.execute_soa_method(
            method_name='createAllocationContext',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'input': input},
            response_cls=GetAllocationWindowResponse,
        )

    @classmethod
    def deleteAllocationLines(cls, allocWindowInput: AllocationWindow, inputAllocationLines: List[AllocationLine]) -> ServiceData:
        """
        The operation is used to delete AllocationLine objects created between BOMLine objects of different product
        structures. This operation needs an AllocationWindow object with an associated AllocationMapRevision object,
        also a list of AllocationLine objects which  are going to be deleted. This operation will return deleted
        AllocationLine objects unique identifier string in the service data as deleted objects. The deletion of
        AllocationLine objects are only allowed  on a Teamcenter Master site and if this operation is attempted from a
        Replica site then an error will be reported and stored as a service data error. If the AllocationWindow has any
        Incremental Context object set, the deletion of AllocationLine will be captured in the IncrementalChange
        business object.
        
        Use cases:
        AllocationLine is a runtime representation of an Allocation object as well as it provides runtime properties.
        It will maintain source and target information for an Allocation. Use this operation to delete Allocation
        between different aspects of a product.
        """
        return cls.execute_soa_method(
            method_name='deleteAllocationLines',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput, 'inputAllocationLines': inputAllocationLines},
            response_cls=ServiceData,
        )

    @classmethod
    def findAllocatedBOMViews(cls, bomView: PSBOMView) -> GetAllocatedBOMViewResponse:
        """
        The operation identifies all the PSBOMView BusinessObjects that are associated to the input PSBOMView Business
        Object in context of any AllocationMap object.
        
        Use cases:
        Find any BOMViews which are associated to the input BOMView for Allocation functionality
        One or more BOMViews in the Product Structure are mapped using Allocation functionality in context of the
        AllocationMap object. Allocation objects are created between the BOMLine objects of a ProductStructure. If the
        user needs to identify the PSBOMView object which has any allocations associated with the input PSBOMView , the
        operation returns such associated PSBOMView Objects.
        """
        return cls.execute_soa_method(
            method_name='findAllocatedBOMViews',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'bomView': bomView},
            response_cls=GetAllocatedBOMViewResponse,
        )

    @classmethod
    def findAllocationContexts(cls, bomViews: List[PSBOMView]) -> ServiceData:
        """
        The operation identifies the AllocationMap Business objects which are used as context for the given input
        PSBOMView Business objects.
        
        Use cases:
        Find AllocationMap objects which are used as context for input PSBOMView objects for Allocation functionality
        Two or more PSBOMView of Product Structure are mapped using Allocation functionality in context of
        AllocationMapRevision object. Allocation objects are created between the BOMLine objects of ProductStructure.
        If user needs to identify for a given list of PSBOMView objects what are all the AllocationMap Business Objects
        which are used as context, the operation returns such AllocationMapRevision Objects.
        """
        return cls.execute_soa_method(
            method_name='findAllocationContexts',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'bomViews': bomViews},
            response_cls=ServiceData,
        )

    @classmethod
    def getBOMViews(cls, allocationContext: AllocationMapRevision) -> ServiceData:
        """
        In allocation functionality in context of an AllocationMapRevision object, allocations are created between  two
        or more product structures. This operation returns the PSBOMView objects associated to the given
        AllocationMapRevision object.
        
        Use cases:
        For the given AllocationMapRevision object, the PSBOMView objects associated to the AllocationWindow object to
        create allocations are identified and returned.
        """
        return cls.execute_soa_method(
            method_name='getBOMViews',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocationContext': allocationContext},
            response_cls=ServiceData,
        )

    @classmethod
    def changeAllocatedBOMWindows(cls, allocWindowInput: AllocationWindow, addBOMWindowList: List[BOMWindow], removeBOMWindowList: List[BOMWindow]) -> GetAllocationWindowResponse:
        """
        The operation allows modifying the BOMWindow business objects associated to the AllocationWindow while working
        with Allocation functionality. Allows user to add new BOMWindow objects to the AllocationWindow and remove any
        existing BOMWindow objects from the AllocationWindow. Once the BOMWindow objects associated are modified, the
        AllocationLine objects for the context of AllocationWindow are returned to the client. Any errors encountered
        will be returned as part of partial errors in ServiceData element.
        
        Use cases:
        To modify the BOMWindows associated to an AllocationWindow
        While working in allocation functionality, if the user wants to work with an additional product structure in
        addition to one which is opened already, or to close some unwanted product structure, the user can achieve this
        using this operation by providing the required BOMWindow business objects in addBOMWindowList and
        removeBOMWindowList.
        """
        return cls.execute_soa_method(
            method_name='changeAllocatedBOMWindows',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput, 'addBOMWindowList': addBOMWindowList, 'removeBOMWindowList': removeBOMWindowList},
            response_cls=GetAllocationWindowResponse,
        )

    @classmethod
    def changeAllocationContext(cls, allocWindowInput: AllocationWindow, allocationContext: AllocationMapRevision, allocationRule: RevisionRule, icContext: ItemRevision) -> GetAllocationWindowResponse:
        """
        This operation is used to change the context object an AllocationMapRevision business object of the input
        AllocationWindow. This operation requires AllocationMapRevision object which user wants to set on
        AllocationWindow. This operation also sets the RevisionRule for AllocationWindow and Incremental Change object
        for the AllocationWindow.
        
        Use cases:
        Sets a new allocation context, Incremental Change context, RevisionRule for an AllocationWindow and reloads the
        allocations for the given set context.
        """
        return cls.execute_soa_method(
            method_name='changeAllocationContext',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput, 'allocationContext': allocationContext, 'allocationRule': allocationRule, 'icContext': icContext},
            response_cls=GetAllocationWindowResponse,
        )

    @classmethod
    def changeICContext(cls, allocWindowInput: AllocationWindow, icContextRev: ItemRevision) -> ServiceData:
        """
        Sets the Incremental Change context for an AllocationWindow. Used to set a new Incremental Change context or
        remove the already set Incremental Change context set on an AllocationWindow. Once the Incremental Change is
        set, the AllocationLine objects for the context of AllocationWindow is returned to the client.
        
        Use cases:
        Use case 1:  Set a new Incremental Change Context on Allocation Window
        - Set a new Incremental Change context for the AllocationWindow. When the Incremental Change context is set on
        an AllocationWindow, the changes made to the AllocationWindow like adding a new Allocation, deleting any
        existing Allocation or modifying any Allocation will be captured in the IC context set. This Incremental Change
        context can be used later to get all the modification made for the given Incremental Change context object.
        
        
        
        Use Case 2:  Remove the Incremental Change Context set on Allocation Window
        - Remove any set Incremental Change context for the AllocationWindow. By passing an NULL input to the operation
        for Incremental Change context will unset the Incremental Change and there after any modifications made to
        AllocationWindow will not be captured incrementally.
        
        """
        return cls.execute_soa_method(
            method_name='changeICContext',
            library='Allocations',
            service_date='2007_01',
            service_name='Allocation',
            params={'allocWindowInput': allocWindowInput, 'icContextRev': icContextRev},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Variant
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetVariabilityInfoResponse(TcBaseObj):
    """
    The 'GetVariabilityObjsResponse' structure represents a list of 'compAndParentVarInfo' objects and the
    'ServiceData' object.
    
    :var compAndParentVarInfo: The list of  'CompAndParentVariabilityInfo' objects containing  variability information
    of the component and its parent.
    :var serviceData: The 'ServiceData' object containing error codes and error messages along with the indices for
    which the getVariabilityInfo operation fails.
    """
    compAndParentVarInfo: List[CompAndParentVariabilityInfo] = ()
    serviceData: ServiceData = None


@dataclass
class VariabilityInfo(TcBaseObj):
    """
    The variability information about each component is provided by way of the 'VariabilityInfo' data structure.
    
    :var variabilityObj: An object pointing to the Variability Expression (VariantExpression with operator code = 2).
    :var variantObj: A Variant object used in the Variability Expression.
    :var comment: An integer value passed through as string which points to the index of the variant value. It is the
    index of variant value from the list of variant values available in the Variant object, which applies to this
    particular Variability Expression.
    """
    variabilityObj: BusinessObject = None
    variantObj: Variant = None
    comment: str = ''


@dataclass
class CompAndParentVariabilityInfo(TcBaseObj):
    """
    The variability information about each component and its parent is provided by way of the
    'CompAndParentVariabilityInfo' data structure.
    
    :var compVarInfo: The list of 'VariabilityInfo' objects containing variability information of component.
    :var parentVarInfo: The list of 'VariabilityInfo' objects containing variability information of the parent of
    component supplied as input.
    """
    compVarInfo: List[VariabilityInfo] = ()
    parentVarInfo: List[VariabilityInfo] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Cfg0AbsValue, Cfg0ConfiguratorPerspective, WorkspaceObject, VariantRule
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class OptionFamily(TcBaseObj):
    """
    A collection of option values.
    
    :var family: The UID of the variant option family.
    :var familyDisplayName: The display name of the variant option family.
    :var familyType: The data type of option values within this family.
    :var isThumbnailDisplay: If true, option values are displayed as tile with thumbnail. If false, option values are
    displayed as simple text. 
    Value of this parameter is governed by preference PCA_Display_Thumbnail.
    :var familyOptions: A map (string, boolean) indicating the various decorations for this family like, if the family
    is complete, single/multi-select, freeform etc. Valid keys are:
    - isComplete: Indicating if at least one option value has been selected in mandatory family.  If true, option
    family is complete. If false, option family is considered incomplete. Optional families are always considered
    complete.
    - isSingleSelect: Indicate if multiple option values in a family allowed to be selected. If true, multiple
    selections are not allowed. If false, multiple selections are allowed.
    
    
    :var optionValues: The list of option values within this family.
    """
    family: str = ''
    familyDisplayName: str = ''
    familyType: str = ''
    isThumbnailDisplay: bool = False
    familyOptions: FamilyOptionMap = None
    optionValues: List[OptionValue] = ()


@dataclass
class OptionGroup(TcBaseObj):
    """
    Collection of variant option families.
    
    :var optGroup: The UID of the variant option group.
    :var groupDisplayName: The display name of the variant option group.
    :var isComplete: Indicates if all required option value selections are complete within this group.
    :var families: The list of option families within this group.
    """
    optGroup: str = ''
    groupDisplayName: str = ''
    isComplete: bool = False
    families: List[OptionFamily] = ()


@dataclass
class OptionValue(TcBaseObj):
    """
    The Variant option value.
    
    :var optValue: The variant option value object. The value is populated when Thumbnail display is ON.
    :var optValueStr: The UID of the variant option value. The value is popouated when Thumbnail display is off.
    :var valueDisplayName: The display name of the variant option value.
    :var selectionState: The state of value selection. 0 - Unselected, 1 - Selected (equal to), and 2 - Negatively
    selected (not equal to).
    :var allowedSelectionStates: A list of all allowable selection states for option value based on current option
    value selections and constraint rules. Valid values in array are 0 - Unselected, 1 - Selected (equal to), and 2 -
    Negatively selected (not equal to). If all states are allowed for an option value, the list will be empty.
    """
    optValue: Cfg0AbsValue = None
    optValueStr: str = ''
    valueDisplayName: str = ''
    selectionState: int = 0
    allowedSelectionStates: List[int] = ()


@dataclass
class OptionValueSelection(TcBaseObj):
    """
    Option value and its selection state as selected by a user.
    
    :var optionValue: The UID of the variant option value ( Cfg0AbsValue ).
    :var selectionState: Variant option value (Cfg0AbsValue) selection state. 0 - Unselected, 1 - Selected (equal to),
    and 2 - Negatively selected (not equal to)
    """
    optionValue: str = ''
    selectionState: int = 0


@dataclass
class SelectionSummary(TcBaseObj):
    """
    Pair of option family name to selected option value.
    
    :var selectionState: Variant option value (Cfg0AbsValue) selection state. Valid values are 1 - Selected (equal to),
    or 2 - Negatively selected (not equal to).
    :var familyDisplayName: Display name of the option family selected. In case of Boolean or multi select families,
    this parameter would contain the option value display name.
    :var valueDisplayValue: Display name of the option value selected. In case of Boolean or multi select families,
    this parameter would be empty.
    """
    selectionState: int = 0
    familyDisplayName: str = ''
    valueDisplayValue: str = ''


@dataclass
class SelectionsSummary(TcBaseObj):
    """
    Summary of all selections in displayable format.
    
    :var summaryOfSelections: The list of all selections per family in user friendly displayable format.
    :var displayName: Display name of selections summary panel.
    :var internalName: Internal name of selections summary panel. This name is used to identify if user has requested
    to view selections summary panel.
    """
    summaryOfSelections: List[SelectionSummary] = ()
    displayName: str = ''
    internalName: str = ''


@dataclass
class VariantConfigurationDataInput(TcBaseObj):
    """
    The input containing the configurator context and variant option value selections to fetch the available
    variability.
    
    :var configContext: The variant configurator context. Valid input types are Item or ItemRevision. This parameter is
    ignored if the 'configPerspective' parameter is passed as not null.
    :var configPerspective: The configurator perspective. When this operation is invoked first time, this parameter
    would be null. But in all subsequent calls, this parameter must be populated with the configurator perspective
    retrieved in the response.
    :var selectedObject: The selected object for which variant configurator context to be retrieved. It can be a
    runtime business object (e.g. instance of Awb0Element) or persistent variant configurable business object (e.g.
    instance of Mdl0ConditionalElement). This parameter is ignored if either configContext or configPerspective is
    passed in input.
    :var optionGroup: The UID of group for which the next valid selections must to be retrieved. If this parameter is
    empty, then option values from model group would be retrieved.
    :var initialVariantRule: An optional initial VariantRule whose expression can be used as input selections for
    computing next valid selections. The parameter is ignored if the 'optionValueSelectionMap' is populated
    :var userSelections: Map(string, OptionValueSelection) of all option values selected by user per family.
    :var guidedConfigurationMode: The configuration mode choice.
    If true, the next valid selections within a group are retrieved after applying configurator constraints. This is
    the Guided configuration mode.
    If false, the next selections within a group are retrieved without applying constraints. This is the Manual
    configuration mode.
    :var switchingToGuidedMode: Flag indicating if we are switching to Guided configuration mode from Manual
    configuration mode. if true, input selections are validated against all constraints. If false, no validation is
    performed.
    """
    configContext: WorkspaceObject = None
    configPerspective: Cfg0ConfiguratorPerspective = None
    selectedObject: BusinessObject = None
    optionGroup: str = ''
    initialVariantRule: VariantRule = None
    userSelections: UserSelectionMap = None
    guidedConfigurationMode: bool = False
    switchingToGuidedMode: bool = False


@dataclass
class VariantConfigurationDataResponse(TcBaseObj):
    """
    The output containing the available variability for the input list of variant option value selections.
    
    :var currentGroup: The variant option group for which the valid selections are retrieved.
    :var allGroups: The list of all variant option groups available in the current Configurator Context.
    :var selectionsSummary: The summary of all selections in displayable format.
    :var isValidConfiguration: Indicates if selections in userSelections map are valid configuration or not. This
    parameter is relevant only when guidedConfigurationMode is true.
    :var configPerspective: The configurator perspective containing all information about the current Configurator
    Context, revision rule and effectivity. All further communications with the server to retrieve variant
    configuration data must use this object.
    :var userSelections: Map(string, OptionValueSelection) of all option values selected by user per family.This map is
    populated from selected option values from initialVariantRule. If initialVariantRule is passed null, this parameter
    would be empty.
    :var serviceData: ServiceData containing partial exceptions, if any.
    """
    currentGroup: str = ''
    allGroups: List[OptionGroup] = ()
    selectionsSummary: SelectionsSummary = None
    isValidConfiguration: bool = False
    configPerspective: Cfg0ConfiguratorPerspective = None
    userSelections: UserSelectionMap = None
    serviceData: ServiceData = None


@dataclass
class CreateCustomVariantRuleResponse(TcBaseObj):
    """
    The output containing custom variant rule.
    
    :var customVariantRule: The adhoc custom variant rule created or updated.
    :var serviceData: ServiceData containing partial exceptions, if any.
    """
    customVariantRule: VariantRule = None
    serviceData: ServiceData = None


"""
Map(string, OptionValueSelection) of all option values selected by user per family.
"""
UserSelectionMap = Dict[str, List[OptionValueSelection]]


"""
A map of ( std::string, bool ) indicating various family options and their values.
"""
FamilyOptionMap = Dict[str, bool]



from __future__ import annotations

from typing import List
from tcsoa.gen.BusinessObjects import RuntimeBusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ClassicBOMLineInfo(TcBaseObj):
    """
    This has the index of input Fnd0BOMLineLite object for which conversion is to be done along with the new BOMLine
    object created by the operation.
    
    :var inputLWBIndex: The index of input Fnd0BOMLineLite object for which conversion was done.
    :var convertedBOMLine: A corresponding BOMLine object that is created from the input Fnd0BOMLineLite object.
    :var lwbChildren: A list of existing Fnd0BOMLineLite child objects of 'convertedBOMLine'.
    """
    inputLWBIndex: int = 0
    convertedBOMLine: RuntimeBusinessObject = None
    lwbChildren: List[RuntimeBusinessObject] = ()


@dataclass
class ConversionResponse(TcBaseObj):
    """
    'ConversionResponse' contains list of 'ClassicBOMLineInfo'.
    
    :var convertedBOMLines: A list of 'ClassicBOMLineInfo'. This has the index of input Fnd0BOMLineLite object for
    which conversion is to be done along with the new BOMLine object created by the operation.
    :var serviceData: All BOMLine parent objects created during the conversion process are added to serviceData.
    All UIDs of deleted Fnd0BOMLineLite objects are added to serviceData.
    """
    convertedBOMLines: List[ClassicBOMLineInfo] = ()
    serviceData: ServiceData = None



from tcsoa.gen.Internal_ActiveWorkspaceVis._2015_03.services import MassiveModelVisualizationService as imp0
from tcsoa.gen.Internal_ActiveWorkspaceVis._2018_05.services import MassiveModelVisualizationService as imp1
from tcsoa.gen.Internal_ActiveWorkspaceVis._2014_11.services import OccurrenceManagementService as imp2
from tcsoa.utils import TcService


class MassiveModelVisualizationService(TcService):
    getIndexedProducts = imp0.getIndexedProducts
    getStructureFiles = imp0.getStructureFiles
    getStructureIdFromRecipe = imp0.getStructureIdFromRecipe
    getStructureIdFromRecipe2 = imp1.getStructureIdFromRecipe2
    groupOccurrencesByProperties = imp0.groupOccurrencesByProperties
    updateDeltaCollection = imp0.updateDeltaCollection


class OccurrenceManagementService(TcService):
    getVisBookmarkInfo = imp2.getVisBookmarkInfo
    saveVisBookmarkInfo = imp2.saveVisBookmarkInfo
